@@ -40,22 +40,20 @@
  * of storage technologies like Fibre Channel over Ethernet (FCoE) and iSCSI.
  **/
 
-G_DEFINE_TYPE_WITH_CODE (NMSettingDcb, nm_setting_dcb, NM_TYPE_SETTING,
-                         _nm_register_setting (DCB, NM_SETTING_PRIORITY_HW_AUX))
-NM_SETTING_REGISTER_TYPE (NM_TYPE_SETTING_DCB)
+G_DEFINE_TYPE (NMSettingDcb, nm_setting_dcb, NM_TYPE_SETTING)
 
 #define NM_SETTING_DCB_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), NM_TYPE_SETTING_DCB, NMSettingDcbPrivate))
 
 typedef struct {
 	NMSettingDcbFlags app_fcoe_flags;
-	gint              app_fcoe_priority;
+	int               app_fcoe_priority;
 	char *            app_fcoe_mode;
 
 	NMSettingDcbFlags app_iscsi_flags;
-	gint              app_iscsi_priority;
+	int               app_iscsi_priority;
 
 	NMSettingDcbFlags app_fip_flags;
-	gint              app_fip_priority;
+	int               app_fip_priority;
 
 	/* Priority Flow Control */
 	NMSettingDcbFlags pfc_flags;
@@ -128,7 +126,7 @@ nm_setting_dcb_get_app_fcoe_flags (NMSettingDcb *setting)
  *
  * Returns: the #NMSettingDcb:app-fcoe-priority property of the setting
  **/
-gint
+int
 nm_setting_dcb_get_app_fcoe_priority (NMSettingDcb *setting)
 {
 	g_return_val_if_fail (NM_IS_SETTING_DCB (setting), 0);
@@ -170,7 +168,7 @@ nm_setting_dcb_get_app_iscsi_flags (NMSettingDcb *setting)
  *
  * Returns: the #NMSettingDcb:app-iscsi-priority property of the setting
  **/
-gint
+int
 nm_setting_dcb_get_app_iscsi_priority (NMSettingDcb *setting)
 {
 	g_return_val_if_fail (NM_IS_SETTING_DCB (setting), 0);
@@ -198,7 +196,7 @@ nm_setting_dcb_get_app_fip_flags (NMSettingDcb *setting)
  *
  * Returns: the #NMSettingDcb:app-fip-priority property of the setting
  **/
-gint
+int
 nm_setting_dcb_get_app_fip_priority (NMSettingDcb *setting)
 {
 	g_return_val_if_fail (NM_IS_SETTING_DCB (setting), 0);
@@ -594,7 +592,7 @@ check_uint_array (const guint *array,
 }
 
 static gboolean
-check_priority (gint val,
+check_priority (int val,
                 NMSettingDcbFlags flags,
                 const char *prop_name,
                 GError **error)
@@ -907,20 +905,20 @@ finalize (GObject *object)
 }
 
 static void
-nm_setting_dcb_class_init (NMSettingDcbClass *setting_class)
+nm_setting_dcb_class_init (NMSettingDcbClass *klass)
 {
-	GObjectClass *object_class = G_OBJECT_CLASS (setting_class);
-	NMSettingClass *parent_class = NM_SETTING_CLASS (setting_class);
+	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+	NMSettingClass *setting_class = NM_SETTING_CLASS (klass);
+	GArray *properties_override = _nm_sett_info_property_override_create_array ();
 
-	g_type_class_add_private (setting_class, sizeof (NMSettingDcbPrivate));
+	g_type_class_add_private (klass, sizeof (NMSettingDcbPrivate));
 
-	/* virtual methods */
 	object_class->set_property = set_property;
 	object_class->get_property = get_property;
 	object_class->finalize = finalize;
-	parent_class->verify       = verify;
 
-	/* Properties */
+	setting_class->verify = verify;
+
 	/**
 	 * NMSettingDcb:app-fcoe-flags:
 	 *
@@ -1099,13 +1097,11 @@ nm_setting_dcb_class_init (NMSettingDcbClass *setting_class)
 		                     G_PARAM_STATIC_STRINGS));
 
 	/**
-	 * NMSettingDcb:priority-flow-control:
+	 * NMSettingDcb:priority-flow-control: (type GArray(gboolean))
 	 *
 	 * An array of 8 boolean values, where the array index corresponds to the User
 	 * Priority (0 - 7) and the value indicates whether or not the corresponding
 	 * priority should transmit priority pause.
-	 *
-	 * Element-type: gboolean
 	 **/
 	/* ---ifcfg-rh---
 	 * property: priority-flow-control
@@ -1121,10 +1117,13 @@ nm_setting_dcb_class_init (NMSettingDcbClass *setting_class)
 		                     G_TYPE_ARRAY,
 		                     G_PARAM_READWRITE |
 		                     G_PARAM_STATIC_STRINGS));
-	_nm_setting_class_transform_property (parent_class, NM_SETTING_DCB_PRIORITY_FLOW_CONTROL,
-	                                      G_VARIANT_TYPE ("au"),
-	                                      _nm_setting_dcb_uint_array_to_dbus,
-	                                      _nm_setting_dcb_uint_array_from_dbus);
+
+	_properties_override_add_transform (properties_override,
+	                                    g_object_class_find_property (G_OBJECT_CLASS (setting_class),
+	                                                                  NM_SETTING_DCB_PRIORITY_FLOW_CONTROL),
+	                                    G_VARIANT_TYPE ("au"),
+	                                    _nm_setting_dcb_uint_array_to_dbus,
+	                                    _nm_setting_dcb_uint_array_from_dbus);
 
 	/**
 	 * NMSettingDcb:priority-group-flags:
@@ -1149,13 +1148,11 @@ nm_setting_dcb_class_init (NMSettingDcbClass *setting_class)
 		                     G_PARAM_STATIC_STRINGS));
 
 	/**
-	 * NMSettingDcb:priority-group-id:
+	 * NMSettingDcb:priority-group-id: (type GArray(guint))
 	 *
 	 * An array of 8 uint values, where the array index corresponds to the User
 	 * Priority (0 - 7) and the value indicates the Priority Group ID.  Allowed
 	 * Priority Group ID values are 0 - 7 or 15 for the unrestricted group.
-	 *
-	 * Element-type: guint
 	 **/
 	/* ---ifcfg-rh---
 	 * property: priority-group-id
@@ -1171,20 +1168,21 @@ nm_setting_dcb_class_init (NMSettingDcbClass *setting_class)
 		                     G_TYPE_ARRAY,
 		                     G_PARAM_READWRITE |
 		                     G_PARAM_STATIC_STRINGS));
-	_nm_setting_class_transform_property (parent_class, NM_SETTING_DCB_PRIORITY_GROUP_ID,
-	                                      G_VARIANT_TYPE ("au"),
-	                                      _nm_setting_dcb_uint_array_to_dbus,
-	                                      _nm_setting_dcb_uint_array_from_dbus);
+
+	_properties_override_add_transform (properties_override,
+	                                    g_object_class_find_property (G_OBJECT_CLASS (setting_class),
+	                                                                  NM_SETTING_DCB_PRIORITY_GROUP_ID),
+	                                    G_VARIANT_TYPE ("au"),
+	                                    _nm_setting_dcb_uint_array_to_dbus,
+	                                    _nm_setting_dcb_uint_array_from_dbus);
 
 	/**
-	 * NMSettingDcb:priority-group-bandwidth:
+	 * NMSettingDcb:priority-group-bandwidth: (type GArray(guint))
 	 *
 	 * An array of 8 uint values, where the array index corresponds to the
 	 * Priority Group ID (0 - 7) and the value indicates the percentage of link
 	 * bandwidth allocated to that group.  Allowed values are 0 - 100, and the
 	 * sum of all values must total 100 percents.
-	 *
-	 * Element-type: guint
 	 **/
 	/* ---ifcfg-rh---
 	 * property: priority-group-bandwidth
@@ -1199,21 +1197,22 @@ nm_setting_dcb_class_init (NMSettingDcbClass *setting_class)
 		                     G_TYPE_ARRAY,
 		                     G_PARAM_READWRITE |
 		                     G_PARAM_STATIC_STRINGS));
-	_nm_setting_class_transform_property (parent_class, NM_SETTING_DCB_PRIORITY_GROUP_BANDWIDTH,
-	                                      G_VARIANT_TYPE ("au"),
-	                                      _nm_setting_dcb_uint_array_to_dbus,
-	                                      _nm_setting_dcb_uint_array_from_dbus);
+
+	_properties_override_add_transform (properties_override,
+	                                    g_object_class_find_property (G_OBJECT_CLASS (setting_class),
+	                                                                  NM_SETTING_DCB_PRIORITY_GROUP_BANDWIDTH),
+	                                    G_VARIANT_TYPE ("au"),
+	                                    _nm_setting_dcb_uint_array_to_dbus,
+	                                    _nm_setting_dcb_uint_array_from_dbus);
 
 	/**
-	 * NMSettingDcb:priority-bandwidth:
+	 * NMSettingDcb:priority-bandwidth: (type GArray(guint))
 	 *
 	 * An array of 8 uint values, where the array index corresponds to the User
 	 * Priority (0 - 7) and the value indicates the percentage of bandwidth of
 	 * the priority's assigned group that the priority may use.  The sum of all
 	 * percentages for priorities which belong to the same group must total 100
 	 * percents.
-	 *
-	 * Element-type: guint
 	 **/
 	/* ---ifcfg-rh---
 	 * property: priority-bandwidth
@@ -1229,19 +1228,20 @@ nm_setting_dcb_class_init (NMSettingDcbClass *setting_class)
 		                     G_TYPE_ARRAY,
 		                     G_PARAM_READWRITE |
 		                     G_PARAM_STATIC_STRINGS));
-	_nm_setting_class_transform_property (parent_class, NM_SETTING_DCB_PRIORITY_BANDWIDTH,
-	                                      G_VARIANT_TYPE ("au"),
-	                                      _nm_setting_dcb_uint_array_to_dbus,
-	                                      _nm_setting_dcb_uint_array_from_dbus);
+
+	_properties_override_add_transform (properties_override,
+	                                    g_object_class_find_property (G_OBJECT_CLASS (setting_class),
+	                                                                  NM_SETTING_DCB_PRIORITY_BANDWIDTH),
+	                                    G_VARIANT_TYPE ("au"),
+	                                    _nm_setting_dcb_uint_array_to_dbus,
+	                                    _nm_setting_dcb_uint_array_from_dbus);
 
 	/**
-	 * NMSettingDcb:priority-strict-bandwidth:
+	 * NMSettingDcb:priority-strict-bandwidth: (type GArray(gboolean))
 	 *
 	 * An array of 8 boolean values, where the array index corresponds to the User
 	 * Priority (0 - 7) and the value indicates whether or not the priority may
 	 * use all of the bandwidth allocated to its assigned group.
-	 *
-	 * Element-type: gboolean
 	 **/
 	/* ---ifcfg-rh---
 	 * property: priority-strict-bandwidth
@@ -1257,19 +1257,20 @@ nm_setting_dcb_class_init (NMSettingDcbClass *setting_class)
 		                     G_TYPE_ARRAY,
 		                     G_PARAM_READWRITE |
 		                     G_PARAM_STATIC_STRINGS));
-	_nm_setting_class_transform_property (parent_class, NM_SETTING_DCB_PRIORITY_STRICT_BANDWIDTH,
-	                                      G_VARIANT_TYPE ("au"),
-	                                      _nm_setting_dcb_uint_array_to_dbus,
-	                                      _nm_setting_dcb_uint_array_from_dbus);
+
+	_properties_override_add_transform (properties_override,
+	                                    g_object_class_find_property (G_OBJECT_CLASS (setting_class),
+	                                                                  NM_SETTING_DCB_PRIORITY_STRICT_BANDWIDTH),
+	                                    G_VARIANT_TYPE ("au"),
+	                                    _nm_setting_dcb_uint_array_to_dbus,
+	                                    _nm_setting_dcb_uint_array_from_dbus);
 
 	/**
-	 * NMSettingDcb:priority-traffic-class:
+	 * NMSettingDcb:priority-traffic-class: (type GArray(guint))
 	 *
 	 * An array of 8 uint values, where the array index corresponds to the User
 	 * Priority (0 - 7) and the value indicates the traffic class (0 - 7) to
 	 * which the priority is mapped.
-	 *
-	 * Element-type: guint
 	 **/
 	/* ---ifcfg-rh---
 	 * property: priority-traffic-class
@@ -1284,8 +1285,14 @@ nm_setting_dcb_class_init (NMSettingDcbClass *setting_class)
 		                     G_TYPE_ARRAY,
 		                     G_PARAM_READWRITE |
 		                     G_PARAM_STATIC_STRINGS));
-	_nm_setting_class_transform_property (parent_class, NM_SETTING_DCB_PRIORITY_TRAFFIC_CLASS,
-	                                      G_VARIANT_TYPE ("au"),
-	                                      _nm_setting_dcb_uint_array_to_dbus,
-	                                      _nm_setting_dcb_uint_array_from_dbus);
+
+	_properties_override_add_transform (properties_override,
+	                                    g_object_class_find_property (G_OBJECT_CLASS (setting_class),
+	                                                                  NM_SETTING_DCB_PRIORITY_TRAFFIC_CLASS),
+	                                    G_VARIANT_TYPE ("au"),
+	                                    _nm_setting_dcb_uint_array_to_dbus,
+	                                    _nm_setting_dcb_uint_array_from_dbus);
+
+	_nm_setting_class_commit_full (setting_class, NM_META_SETTING_TYPE_DCB,
+	                               NULL, properties_override);
 }
