@@ -59,7 +59,7 @@ static const char* _link_watcher_name[] = {
 struct NMTeamLinkWatcher {
 	guint refcount;
 
-	guint8 type;	/* LinkWatcherTypes */
+	guint8 type; /* LinkWatcherTypes */
 
 	/*
 	 * The union is constructed in order to allow mapping the options of all the
@@ -115,19 +115,19 @@ struct NMTeamLinkWatcher {
  *
  * Returns: (transfer full): the new #NMTeamLinkWatcher object
  *
- * Since: 1.10.2
+ * Since: 1.12
  **/
 NMTeamLinkWatcher *
-nm_team_link_watcher_new_ethtool (gint delay_up,
-                                  gint delay_down,
+nm_team_link_watcher_new_ethtool (int delay_up,
+                                  int delay_down,
                                   GError **error)
 {
 	NMTeamLinkWatcher *watcher;
 	const char *val_fail = NULL;
 
-	if (delay_up < 0 || delay_up > G_MAXINT32)
+	if (delay_up < 0 || !_NM_INT_LE_MAXINT32 (delay_up))
 		val_fail = "delay-up";
-	if (delay_down < 0 || delay_down > G_MAXINT32)
+	if (delay_down < 0 || !_NM_INT_LE_MAXINT32 (delay_down))
 		val_fail = "delay-down";
 	if (val_fail) {
 		g_set_error (error, NM_CONNECTION_ERROR, NM_CONNECTION_ERROR_FAILED,
@@ -158,12 +158,12 @@ nm_team_link_watcher_new_ethtool (gint delay_up,
  *
  * Returns: (transfer full): the new #NMTeamLinkWatcher object, or %NULL on error
  *
- * Since: 1.10.2
+ * Since: 1.12
  **/
 NMTeamLinkWatcher *
-nm_team_link_watcher_new_nsna_ping (gint init_wait,
-                                    gint interval,
-                                    gint missed_max,
+nm_team_link_watcher_new_nsna_ping (int init_wait,
+                                    int interval,
+                                    int missed_max,
                                     const char *target_host,
                                     GError **error)
 {
@@ -182,11 +182,11 @@ nm_team_link_watcher_new_nsna_ping (gint init_wait,
 		return NULL;
 	}
 
-	if (init_wait < 0 || init_wait > G_MAXINT32)
+	if (init_wait < 0 || !_NM_INT_LE_MAXINT32 (init_wait))
 		val_fail = "init-wait";
-	if (interval < 0 || interval > G_MAXINT32)
+	if (interval < 0 || !_NM_INT_LE_MAXINT32 (interval))
 		val_fail = "interval";
-	if (missed_max < 0 || missed_max > G_MAXINT32)
+	if (missed_max < 0 || !_NM_INT_LE_MAXINT32 (missed_max))
 		val_fail = "missed-max";
 	if (val_fail) {
 		g_set_error (error, NM_CONNECTION_ERROR, NM_CONNECTION_ERROR_FAILED,
@@ -222,12 +222,12 @@ nm_team_link_watcher_new_nsna_ping (gint init_wait,
  *
  * Returns: (transfer full): the new #NMTeamLinkWatcher object, or %NULL on error
  *
- * Since: 1.10.2
+ * Since: 1.12
  **/
 NMTeamLinkWatcher *
-nm_team_link_watcher_new_arp_ping (gint init_wait,
-                                   gint interval,
-                                   gint missed_max,
+nm_team_link_watcher_new_arp_ping (int init_wait,
+                                   int interval,
+                                   int missed_max,
                                    const char *target_host,
                                    const char *source_host,
                                    NMTeamLinkWatcherArpPingFlags flags,
@@ -255,11 +255,11 @@ nm_team_link_watcher_new_arp_ping (gint init_wait,
 		return NULL;
 	}
 
-	if (init_wait < 0 || init_wait > G_MAXINT32)
+	if (init_wait < 0 || !_NM_INT_LE_MAXINT32 (init_wait))
 		val_fail = "init-wait";
-	if (interval < 0 || interval > G_MAXINT32)
+	if (interval < 0 || !_NM_INT_LE_MAXINT32 (interval))
 		val_fail = "interval";
-	if (missed_max < 0 || missed_max > G_MAXINT32)
+	if (missed_max < 0 || !_NM_INT_LE_MAXINT32 (missed_max))
 		val_fail = "missed-max";
 	if (val_fail) {
 		g_set_error (error, NM_CONNECTION_ERROR, NM_CONNECTION_ERROR_FAILED,
@@ -287,7 +287,7 @@ nm_team_link_watcher_new_arp_ping (gint init_wait,
  *
  * Increases the reference count of the object.
  *
- * Since: 1.10.2
+ * Since: 1.12
  **/
 void
 nm_team_link_watcher_ref (NMTeamLinkWatcher *watcher){
@@ -303,7 +303,7 @@ nm_team_link_watcher_ref (NMTeamLinkWatcher *watcher){
  * Decreases the reference count of the object.  If the reference count
  * reaches zero, the object will be destroyed.
  *
- * Since: 1.10.2
+ * Since: 1.12
  **/
 void
 nm_team_link_watcher_unref (NMTeamLinkWatcher *watcher)
@@ -328,7 +328,7 @@ nm_team_link_watcher_unref (NMTeamLinkWatcher *watcher)
  *
  * Returns: %TRUE if the objects contain the same values, %FALSE if they do not.
  *
- * Since: 1.10.2
+ * Since: 1.12
  **/
 gboolean
 nm_team_link_watcher_equal (NMTeamLinkWatcher *watcher, NMTeamLinkWatcher *other)
@@ -356,7 +356,7 @@ nm_team_link_watcher_equal (NMTeamLinkWatcher *watcher, NMTeamLinkWatcher *other
  *
  * Returns: (transfer full): a copy of @watcher
  *
- * Since: 1.10.2
+ * Since: 1.12
  **/
 NMTeamLinkWatcher *
 nm_team_link_watcher_dup (NMTeamLinkWatcher *watcher)
@@ -396,7 +396,7 @@ nm_team_link_watcher_dup (NMTeamLinkWatcher *watcher)
  *
  * Gets the name of the link watcher to be used.
  *
- * Since: 1.10.2
+ * Since: 1.12
  **/
 const char *
 nm_team_link_watcher_get_name (NMTeamLinkWatcher *watcher)
@@ -413,7 +413,7 @@ nm_team_link_watcher_get_name (NMTeamLinkWatcher *watcher)
  * Gets the delay_up interval (in milliseconds) that elapses between the link
  * coming up and the runner beeing notified about it.
  *
- * Since: 1.10.2
+ * Since: 1.12
  **/
 int
 nm_team_link_watcher_get_delay_up (NMTeamLinkWatcher *watcher)
@@ -432,7 +432,7 @@ nm_team_link_watcher_get_delay_up (NMTeamLinkWatcher *watcher)
  * Gets the delay_down interval (in milliseconds) that elapses between the link
  * going down and the runner beeing notified about it.
  *
- * Since: 1.10.2
+ * Since: 1.12
  **/
 int
 nm_team_link_watcher_get_delay_down (NMTeamLinkWatcher *watcher)
@@ -451,7 +451,7 @@ nm_team_link_watcher_get_delay_down (NMTeamLinkWatcher *watcher)
  * Gets the init_wait interval (in milliseconds) that the team slave should
  * wait before sending the first packet to the target host.
  *
- * Since: 1.10.2
+ * Since: 1.12
  **/
 int
 nm_team_link_watcher_get_init_wait (NMTeamLinkWatcher *watcher)
@@ -472,7 +472,7 @@ nm_team_link_watcher_get_init_wait (NMTeamLinkWatcher *watcher)
  * Gets the interval (in milliseconds) that the team slave should wait between
  * sending two check packets to the target host.
  *
- * Since: 1.10.2
+ * Since: 1.12
  **/
 int
 nm_team_link_watcher_get_interval (NMTeamLinkWatcher *watcher)
@@ -492,7 +492,7 @@ nm_team_link_watcher_get_interval (NMTeamLinkWatcher *watcher)
  *
  * Gets the number of missed replies after which the link is considered down.
  *
- * Since: 1.10.2
+ * Since: 1.12
  **/
 int
 nm_team_link_watcher_get_missed_max (NMTeamLinkWatcher *watcher)
@@ -513,7 +513,7 @@ nm_team_link_watcher_get_missed_max (NMTeamLinkWatcher *watcher)
  * Gets the host name/ip address to be used as destination for the link probing
  * packets.
  *
- * Since: 1.10.2
+ * Since: 1.12
  **/
 const char *
 nm_team_link_watcher_get_target_host (NMTeamLinkWatcher *watcher)
@@ -529,7 +529,7 @@ nm_team_link_watcher_get_target_host (NMTeamLinkWatcher *watcher)
  *
  * Gets the ip address to be used as source for the link probing packets.
  *
- * Since: 1.10.2
+ * Since: 1.12
  **/
 const char *
 nm_team_link_watcher_get_source_host (NMTeamLinkWatcher *watcher)
@@ -545,7 +545,7 @@ nm_team_link_watcher_get_source_host (NMTeamLinkWatcher *watcher)
  *
  * Gets the arp ping watcher flags.
  *
- * Since: 1.10.2
+ * Since: 1.12
  **/
 NMTeamLinkWatcherArpPingFlags
 nm_team_link_watcher_get_flags (NMTeamLinkWatcher *watcher)
@@ -557,27 +557,25 @@ nm_team_link_watcher_get_flags (NMTeamLinkWatcher *watcher)
 
 /*****************************************************************************/
 
-G_DEFINE_TYPE_WITH_CODE (NMSettingTeam, nm_setting_team, NM_TYPE_SETTING,
-                         _nm_register_setting (TEAM, NM_SETTING_PRIORITY_HW_BASE))
-NM_SETTING_REGISTER_TYPE (NM_TYPE_SETTING_TEAM)
+G_DEFINE_TYPE (NMSettingTeam, nm_setting_team, NM_TYPE_SETTING)
 
 #define NM_SETTING_TEAM_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), NM_TYPE_SETTING_TEAM, NMSettingTeamPrivate))
 
 typedef struct {
 	char *config;
-	gint notify_peers_count;
-	gint notify_peers_interval;
-	gint mcast_rejoin_count;
-	gint mcast_rejoin_interval;
+	int notify_peers_count;
+	int notify_peers_interval;
+	int mcast_rejoin_count;
+	int mcast_rejoin_interval;
 	char *runner;
 	char *runner_hwaddr_policy;
 	GPtrArray *runner_tx_hash;
 	char *runner_tx_balancer;
-	gint runner_tx_balancer_interval;
+	int runner_tx_balancer_interval;
 	gboolean runner_active;
 	gboolean runner_fast_rate;
-	gint runner_sys_prio;
-	gint runner_min_ports;
+	int runner_sys_prio;
+	int runner_min_ports;
 	char *runner_agg_select_policy;
 	GPtrArray *link_watchers; /* Array of NMTeamLinkWatcher */
 } NMSettingTeamPrivate;
@@ -659,9 +657,9 @@ nm_setting_team_get_config (NMSettingTeam *setting)
  *
  * Returns: the ##NMSettingTeam:notify-peers-count property of the setting
  *
- * Since: 1.10.2
+ * Since: 1.12
  **/
-gint
+int
 nm_setting_team_get_notify_peers_count (NMSettingTeam *setting)
 {
 	g_return_val_if_fail (NM_IS_SETTING_TEAM (setting), 0);
@@ -675,9 +673,9 @@ nm_setting_team_get_notify_peers_count (NMSettingTeam *setting)
  *
  * Returns: the ##NMSettingTeam:notify-peers-interval property of the setting
  *
- * Since: 1.10.2
+ * Since: 1.12
  **/
-gint
+int
 nm_setting_team_get_notify_peers_interval (NMSettingTeam *setting)
 {
 	g_return_val_if_fail (NM_IS_SETTING_TEAM (setting), 0);
@@ -691,9 +689,9 @@ nm_setting_team_get_notify_peers_interval (NMSettingTeam *setting)
  *
  * Returns: the ##NMSettingTeam:mcast-rejoin-count property of the setting
  *
- * Since: 1.10.2
+ * Since: 1.12
  **/
-gint
+int
 nm_setting_team_get_mcast_rejoin_count (NMSettingTeam *setting)
 {
 	g_return_val_if_fail (NM_IS_SETTING_TEAM (setting), 0);
@@ -707,9 +705,9 @@ nm_setting_team_get_mcast_rejoin_count (NMSettingTeam *setting)
  *
  * Returns: the ##NMSettingTeam:mcast-rejoin-interval property of the setting
  *
- * Since: 1.10.2
+ * Since: 1.12
  **/
-gint
+int
 nm_setting_team_get_mcast_rejoin_interval (NMSettingTeam *setting)
 {
 	g_return_val_if_fail (NM_IS_SETTING_TEAM (setting), 0);
@@ -723,7 +721,7 @@ nm_setting_team_get_mcast_rejoin_interval (NMSettingTeam *setting)
  *
  * Returns: the ##NMSettingTeam:runner property of the setting
  *
- * Since: 1.10.2
+ * Since: 1.12
  **/
 const char *
 nm_setting_team_get_runner (NMSettingTeam *setting)
@@ -739,7 +737,7 @@ nm_setting_team_get_runner (NMSettingTeam *setting)
  *
  * Returns: the ##NMSettingTeam:runner-hwaddr-policy property of the setting
  *
- * Since: 1.10.2
+ * Since: 1.12
  **/
 const char *
 nm_setting_team_get_runner_hwaddr_policy (NMSettingTeam *setting)
@@ -755,7 +753,7 @@ nm_setting_team_get_runner_hwaddr_policy (NMSettingTeam *setting)
  *
  * Returns: the ##NMSettingTeam:runner-tx-balancer property of the setting
  *
- * Since: 1.10.2
+ * Since: 1.12
  **/
 const char *
 nm_setting_team_get_runner_tx_balancer (NMSettingTeam *setting)
@@ -771,9 +769,9 @@ nm_setting_team_get_runner_tx_balancer (NMSettingTeam *setting)
  *
  * Returns: the ##NMSettingTeam:runner-tx-balancer_interval property of the setting
  *
- * Since: 1.10.2
+ * Since: 1.12
  **/
-gint
+int
 nm_setting_team_get_runner_tx_balancer_interval (NMSettingTeam *setting)
 {
 	g_return_val_if_fail (NM_IS_SETTING_TEAM (setting), 0);
@@ -787,7 +785,7 @@ nm_setting_team_get_runner_tx_balancer_interval (NMSettingTeam *setting)
  *
  * Returns: the ##NMSettingTeam:runner_active property of the setting
  *
- * Since: 1.10.2
+ * Since: 1.12
  **/
 gboolean
 nm_setting_team_get_runner_active (NMSettingTeam *setting)
@@ -803,7 +801,7 @@ nm_setting_team_get_runner_active (NMSettingTeam *setting)
  *
  * Returns: the ##NMSettingTeam:runner-fast-rate property of the setting
  *
- * Since: 1.10.2
+ * Since: 1.12
  **/
 gboolean
 nm_setting_team_get_runner_fast_rate (NMSettingTeam *setting)
@@ -819,9 +817,9 @@ nm_setting_team_get_runner_fast_rate (NMSettingTeam *setting)
  *
  * Returns: the ##NMSettingTeam:runner-sys-prio property of the setting
  *
- * Since: 1.10.2
+ * Since: 1.12
  **/
-gint
+int
 nm_setting_team_get_runner_sys_prio (NMSettingTeam *setting)
 {
 	g_return_val_if_fail (NM_IS_SETTING_TEAM (setting), 0);
@@ -835,9 +833,9 @@ nm_setting_team_get_runner_sys_prio (NMSettingTeam *setting)
  *
  * Returns: the ##NMSettingTeam:runner-min-ports property of the setting
  *
- * Since: 1.10.2
+ * Since: 1.12
  **/
-gint
+int
 nm_setting_team_get_runner_min_ports (NMSettingTeam *setting)
 {
 	g_return_val_if_fail (NM_IS_SETTING_TEAM (setting), 0);
@@ -851,7 +849,7 @@ nm_setting_team_get_runner_min_ports (NMSettingTeam *setting)
  *
  * Returns: the ##NMSettingTeam:runner-agg-select-policy property of the setting
  *
- * Since: 1.10.2
+ * Since: 1.12
  **/
 const char *
 nm_setting_team_get_runner_agg_select_policy (NMSettingTeam *setting)
@@ -870,7 +868,7 @@ nm_setting_team_get_runner_agg_select_policy (NMSettingTeam *setting)
  *
  * Returns: %TRUE if the txhash element was found and removed; %FALSE if it was not.
  *
- * Since: 1.10.2
+ * Since: 1.12
  **/
 gboolean
 nm_setting_team_remove_runner_tx_hash_by_value (NMSettingTeam *setting,
@@ -899,7 +897,7 @@ nm_setting_team_remove_runner_tx_hash_by_value (NMSettingTeam *setting,
  *
  * Returns: the number of elements in txhash
  *
- * Since: 1.10.2
+ * Since: 1.12
  **/
 guint
 nm_setting_team_get_num_runner_tx_hash (NMSettingTeam *setting)
@@ -918,7 +916,7 @@ nm_setting_team_get_num_runner_tx_hash (NMSettingTeam *setting)
  *
  * Returns: the txhash element at index @idx
  *
- * Since: 1.10.2
+ * Since: 1.12
  **/
 const char *
 nm_setting_team_get_runner_tx_hash (NMSettingTeam *setting, guint idx)
@@ -938,7 +936,7 @@ nm_setting_team_get_runner_tx_hash (NMSettingTeam *setting, guint idx)
  *
  * Removes the txhash element at index @idx.
  *
- * Since: 1.10.2
+ * Since: 1.12
  **/
 void
 nm_setting_team_remove_runner_tx_hash (NMSettingTeam *setting, guint idx)
@@ -962,7 +960,7 @@ nm_setting_team_remove_runner_tx_hash (NMSettingTeam *setting, guint idx)
  * Returns: %TRUE if the txhash element was added; %FALSE if the element
  * was already knnown.
  *
- * Since: 1.10.2
+ * Since: 1.12
  **/
 gboolean
 nm_setting_team_add_runner_tx_hash (NMSettingTeam *setting, const char *txhash)
@@ -992,7 +990,7 @@ nm_setting_team_add_runner_tx_hash (NMSettingTeam *setting, const char *txhash)
  *
  * Returns: the number of configured link watchers
  *
- * Since: 1.10.2
+ * Since: 1.12
  **/
 guint
 nm_setting_team_get_num_link_watchers (NMSettingTeam *setting)
@@ -1011,7 +1009,7 @@ nm_setting_team_get_num_link_watchers (NMSettingTeam *setting)
  *
  * Returns: (transfer none): the link watcher at index @idx.
  *
- * Since: 1.10.2
+ * Since: 1.12
  **/
 NMTeamLinkWatcher *
 nm_setting_team_get_link_watcher (NMSettingTeam *setting, guint idx)
@@ -1034,7 +1032,7 @@ nm_setting_team_get_link_watcher (NMSettingTeam *setting, guint idx)
  * Returns: %TRUE if the link watcher is added; %FALSE if an identical link
  * watcher was already there.
  *
- * Since: 1.10.2
+ * Since: 1.12
  **/
 gboolean
 nm_setting_team_add_link_watcher (NMSettingTeam *setting,
@@ -1063,7 +1061,7 @@ nm_setting_team_add_link_watcher (NMSettingTeam *setting,
  *
  * Removes the link watcher at index #idx.
  *
- * Since: 1.10.2
+ * Since: 1.12
  **/
 void
 nm_setting_team_remove_link_watcher (NMSettingTeam *setting, guint idx)
@@ -1086,7 +1084,7 @@ nm_setting_team_remove_link_watcher (NMSettingTeam *setting, guint idx)
  *
  * Returns: %TRUE if the link watcher was found and removed, %FALSE otherwise.
  *
- * Since: 1.10.2
+ * Since: 1.12
  **/
 gboolean
 nm_setting_team_remove_link_watcher_by_value (NMSettingTeam *setting,
@@ -1113,7 +1111,7 @@ nm_setting_team_remove_link_watcher_by_value (NMSettingTeam *setting,
  *
  * Removes all configured link watchers.
  *
- * Since: 1.10.2
+ * Since: 1.12
  **/
 void
 nm_setting_team_clear_link_watchers (NMSettingTeam *setting) {
@@ -1121,8 +1119,10 @@ nm_setting_team_clear_link_watchers (NMSettingTeam *setting) {
 
 	g_return_if_fail (NM_IS_SETTING_TEAM (setting));
 
-	g_ptr_array_set_size (priv->link_watchers, 0);
-	g_object_notify (G_OBJECT (setting), NM_SETTING_TEAM_LINK_WATCHERS);
+	if (priv->link_watchers->len != 0) {
+		g_ptr_array_set_size (priv->link_watchers, 0);
+		g_object_notify (G_OBJECT (setting), NM_SETTING_TEAM_LINK_WATCHERS);
+	}
 }
 
 static GVariant *
@@ -1237,7 +1237,7 @@ compare_property (NMSetting *setting,
                   NMSettingCompareFlags flags)
 {
 	NMSettingTeamPrivate *a_priv, *b_priv;
-	NMSettingClass *parent_class;
+	NMSettingClass *setting_class;
 	guint i, j;
 
 	/* If we are trying to match a connection in order to assume it (and thus
@@ -1270,9 +1270,8 @@ compare_property (NMSetting *setting,
 		return TRUE;
 	}
 
-	/* Otherwise chain up to parent to handle generic compare */
-	parent_class = NM_SETTING_CLASS (nm_setting_team_parent_class);
-	return parent_class->compare_property (setting, other, prop_spec, flags);
+	setting_class = NM_SETTING_CLASS (nm_setting_team_parent_class);
+	return setting_class->compare_property (setting, other, prop_spec, flags);
 }
 
 static void
@@ -1304,7 +1303,6 @@ finalize (GObject *object)
 	G_OBJECT_CLASS (nm_setting_team_parent_class)->finalize (object);
 }
 
-
 #define JSON_TO_VAL(typ, id)   _nm_utils_json_extract_##typ (priv->config, _prop_to_keys[id], FALSE)
 
 static void
@@ -1558,21 +1556,21 @@ get_property (GObject *object, guint prop_id,
 }
 
 static void
-nm_setting_team_class_init (NMSettingTeamClass *setting_class)
+nm_setting_team_class_init (NMSettingTeamClass *klass)
 {
-	GObjectClass *object_class = G_OBJECT_CLASS (setting_class);
-	NMSettingClass *parent_class = NM_SETTING_CLASS (setting_class);
+	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+	NMSettingClass *setting_class = NM_SETTING_CLASS (klass);
+	GArray *properties_override = _nm_sett_info_property_override_create_array ();
 
-	g_type_class_add_private (setting_class, sizeof (NMSettingTeamPrivate));
+	g_type_class_add_private (klass, sizeof (NMSettingTeamPrivate));
 
-	/* virtual methods */
 	object_class->set_property     = set_property;
 	object_class->get_property     = get_property;
 	object_class->finalize         = finalize;
-	parent_class->compare_property = compare_property;
-	parent_class->verify           = verify;
 
-	/* Properties */
+	setting_class->compare_property = compare_property;
+	setting_class->verify           = verify;
+
 	/**
 	 * NMSettingTeam:config:
 	 *
@@ -1600,7 +1598,7 @@ nm_setting_team_class_init (NMSettingTeamClass *setting_class)
 	 *
 	 * Corresponds to the teamd notify_peers.count.
 	 *
-	 * Since: 1.10.2
+	 * Since: 1.12
 	 **/
 	g_object_class_install_property
 		(object_class, PROP_NOTIFY_PEERS_COUNT,
@@ -1614,7 +1612,7 @@ nm_setting_team_class_init (NMSettingTeamClass *setting_class)
 	 *
 	 * Corresponds to the teamd notify_peers.interval.
 	 *
-	 * Since: 1.10.2
+	 * Since: 1.12
 	 **/
 	g_object_class_install_property
 		(object_class, PROP_NOTIFY_PEERS_INTERVAL,
@@ -1628,7 +1626,7 @@ nm_setting_team_class_init (NMSettingTeamClass *setting_class)
 	 *
 	 * Corresponds to the teamd mcast_rejoin.count.
 	 *
-	 * Since: 1.10.2
+	 * Since: 1.12
 	 **/
 	g_object_class_install_property
 		(object_class, PROP_MCAST_REJOIN_COUNT,
@@ -1642,7 +1640,7 @@ nm_setting_team_class_init (NMSettingTeamClass *setting_class)
 	 *
 	 * Corresponds to the teamd mcast_rejoin.interval.
 	 *
-	 * Since: 1.10.2
+	 * Since: 1.12
 	 **/
 	g_object_class_install_property
 		(object_class, PROP_MCAST_REJOIN_INTERVAL,
@@ -1656,13 +1654,13 @@ nm_setting_team_class_init (NMSettingTeamClass *setting_class)
 	 *
 	 * Corresponds to the teamd runner.name.
 	 * Permitted values are: "roundrobin", "broadcast", "activebackup",
-	 * "loadbalance", "lacp".
+	 * "loadbalance", "lacp", "random".
 	 * When setting the runner, all the properties specific to the runner
 	 * will be reset to the default value; all the properties specific to
 	 * other runners will be set to an empty value (or if not possible to
 	 * a default value).
 	 *
-	 * Since: 1.10.2
+	 * Since: 1.12
 	 **/
 	g_object_class_install_property
 		(object_class, PROP_RUNNER,
@@ -1676,7 +1674,7 @@ nm_setting_team_class_init (NMSettingTeamClass *setting_class)
 	 *
 	 * Corresponds to the teamd runner.hwaddr_policy.
 	 *
-	 * Since: 1.10.2
+	 * Since: 1.12
 	 **/
 	g_object_class_install_property
 		(object_class, PROP_RUNNER_HWADDR_POLICY,
@@ -1690,7 +1688,7 @@ nm_setting_team_class_init (NMSettingTeamClass *setting_class)
 	 *
 	 * Corresponds to the teamd runner.tx_hash.
 	 *
-	 * Since: 1.10.2
+	 * Since: 1.12
 	 **/
 	g_object_class_install_property
 		(object_class, PROP_RUNNER_TX_HASH,
@@ -1705,7 +1703,7 @@ nm_setting_team_class_init (NMSettingTeamClass *setting_class)
 	 *
 	 * Corresponds to the teamd runner.tx_balancer.name.
 	 *
-	 * Since: 1.10.2
+	 * Since: 1.12
 	 **/
 	g_object_class_install_property
 		(object_class, PROP_RUNNER_TX_BALANCER,
@@ -1719,7 +1717,7 @@ nm_setting_team_class_init (NMSettingTeamClass *setting_class)
 	 *
 	 * Corresponds to the teamd runner.tx_balancer.interval.
 	 *
-	 * Since: 1.10.2
+	 * Since: 1.12
 	 **/
 	g_object_class_install_property
 		(object_class, PROP_RUNNER_TX_BALANCER_INTERVAL,
@@ -1733,7 +1731,7 @@ nm_setting_team_class_init (NMSettingTeamClass *setting_class)
 	 *
 	 * Corresponds to the teamd runner.active.
 	 *
-	 * Since: 1.10.2
+	 * Since: 1.12
 	 **/
 	g_object_class_install_property
 		(object_class, PROP_RUNNER_ACTIVE,
@@ -1747,7 +1745,7 @@ nm_setting_team_class_init (NMSettingTeamClass *setting_class)
 	 *
 	 * Corresponds to the teamd runner.fast_rate.
 	 *
-	 * Since: 1.10.2
+	 * Since: 1.12
 	 **/
 	g_object_class_install_property
 		(object_class, PROP_RUNNER_FAST_RATE,
@@ -1761,7 +1759,7 @@ nm_setting_team_class_init (NMSettingTeamClass *setting_class)
 	 *
 	 * Corresponds to the teamd runner.sys_prio.
 	 *
-	 * Since: 1.10.2
+	 * Since: 1.12
 	 **/
 	g_object_class_install_property
 		(object_class, PROP_RUNNER_SYS_PRIO,
@@ -1775,7 +1773,7 @@ nm_setting_team_class_init (NMSettingTeamClass *setting_class)
 	 *
 	 * Corresponds to the teamd runner.min_ports.
 	 *
-	 * Since: 1.10.2
+	 * Since: 1.12
 	 **/
 	g_object_class_install_property
 		(object_class, PROP_RUNNER_MIN_PORTS,
@@ -1789,7 +1787,7 @@ nm_setting_team_class_init (NMSettingTeamClass *setting_class)
 	 *
 	 * Corresponds to the teamd runner.agg_select_policy.
 	 *
-	 * Since: 1.10.2
+	 * Since: 1.12
 	 **/
 	g_object_class_install_property
 		(object_class, PROP_RUNNER_AGG_SELECT_POLICY,
@@ -1799,7 +1797,7 @@ nm_setting_team_class_init (NMSettingTeamClass *setting_class)
 		                      G_PARAM_STATIC_STRINGS));
 
 	/**
-	 * NMSettingTeam:link-watchers:
+	 * NMSettingTeam:link-watchers: (type GPtrArray(NMTeamLinkWatcher))
 	 *
 	 * Link watchers configuration for the connection: each link watcher is
 	 * defined by a dictionary, whose keys depend upon the selected link
@@ -1810,8 +1808,7 @@ nm_setting_team_class_init (NMSettingTeamClass *setting_class)
 	 * arp_ping: all the ones in nsna_ping and 'source-host', 'validate-active',
 	 * 'validate-incative', 'send-always'. See teamd.conf man for more details.
 	 *
-	 * Element-Type: NMTeamLinkWatcher
-	 * Since: 1.10.2
+	 * Since: 1.12
 	 **/
 	g_object_class_install_property
 		(object_class, PROP_LINK_WATCHERS,
@@ -1819,11 +1816,13 @@ nm_setting_team_class_init (NMSettingTeamClass *setting_class)
 		                     G_TYPE_PTR_ARRAY,
 		                     G_PARAM_READWRITE |
 		                     G_PARAM_STATIC_STRINGS));
-	_nm_setting_class_transform_property (parent_class,
-	                                     NM_SETTING_TEAM_LINK_WATCHERS,
-	                                     G_VARIANT_TYPE ("aa{sv}"),
-	                                     team_link_watchers_to_dbus,
-	                                     team_link_watchers_from_dbus);
+
+	_properties_override_add_transform (properties_override,
+	                                    g_object_class_find_property (G_OBJECT_CLASS (setting_class),
+	                                                                  NM_SETTING_TEAM_LINK_WATCHERS),
+	                                    G_VARIANT_TYPE ("aa{sv}"),
+	                                    team_link_watchers_to_dbus,
+	                                    team_link_watchers_from_dbus);
 
 	/* ---dbus---
 	 * property: interface-name
@@ -1833,8 +1832,12 @@ nm_setting_team_class_init (NMSettingTeamClass *setting_class)
 	 *   team's interface name.
 	 * ---end---
 	 */
-	_nm_setting_class_add_dbus_only_property (parent_class, "interface-name",
-	                                          G_VARIANT_TYPE_STRING,
-	                                          _nm_setting_get_deprecated_virtual_interface_name,
-	                                          NULL);
+	_properties_override_add_dbus_only (properties_override,
+	                                    "interface-name",
+	                                    G_VARIANT_TYPE_STRING,
+	                                    _nm_setting_get_deprecated_virtual_interface_name,
+	                                    NULL);
+
+	_nm_setting_class_commit_full (setting_class, NM_META_SETTING_TYPE_TEAM,
+	                               NULL, properties_override);
 }
