@@ -27,7 +27,7 @@
 #error "Only <NetworkManager.h> can be included directly."
 #endif
 
-#include <nm-setting.h>
+#include "nm-setting.h"
 
 G_BEGIN_DECLS
 
@@ -49,6 +49,8 @@ G_BEGIN_DECLS
 
 /**
  * NMSettingVpn:
+ *
+ * VPN Settings
  */
 struct _NMSettingVpn {
 	NMSetting parent;
@@ -88,6 +90,9 @@ gboolean          nm_setting_vpn_remove_data_item  (NMSettingVpn *setting,
 void              nm_setting_vpn_foreach_data_item (NMSettingVpn *setting,
                                                     NMVpnIterFunc func,
                                                     gpointer user_data);
+NM_AVAILABLE_IN_1_12
+const char **     nm_setting_vpn_get_data_keys     (NMSettingVpn *setting,
+                                                    guint *out_length);
 
 guint32           nm_setting_vpn_get_num_secrets   (NMSettingVpn *setting);
 void              nm_setting_vpn_add_secret        (NMSettingVpn *setting,
@@ -100,6 +105,10 @@ gboolean          nm_setting_vpn_remove_secret     (NMSettingVpn *setting,
 void              nm_setting_vpn_foreach_secret    (NMSettingVpn *setting,
                                                     NMVpnIterFunc func,
                                                     gpointer user_data);
+NM_AVAILABLE_IN_1_12
+const char **     nm_setting_vpn_get_secret_keys   (NMSettingVpn *setting,
+                                                    guint *out_length);
+
 NM_AVAILABLE_IN_1_2
 guint32           nm_setting_vpn_get_timeout       (NMSettingVpn *setting);
 
