@@ -29,17 +29,17 @@
 
 #if !defined (NM_VERSION_MIN_REQUIRED) || (NM_VERSION_MIN_REQUIRED == 0)
 # undef NM_VERSION_MIN_REQUIRED
-# define NM_VERSION_MIN_REQUIRED (NM_VERSION_CUR_STABLE)
+# define NM_VERSION_MIN_REQUIRED (NM_API_VERSION)
 #endif
 
 #if !defined (NM_VERSION_MAX_ALLOWED) || (NM_VERSION_MAX_ALLOWED == 0)
 # undef NM_VERSION_MAX_ALLOWED
-# define NM_VERSION_MAX_ALLOWED (NM_VERSION_CUR_STABLE)
+# define NM_VERSION_MAX_ALLOWED (NM_API_VERSION)
 #endif
 
 /* sanity checks */
-#if NM_VERSION_MIN_REQUIRED > NM_VERSION_NEXT_STABLE
-#error "NM_VERSION_MIN_REQUIRED must be <= NM_VERSION_NEXT_STABLE"
+#if NM_VERSION_MIN_REQUIRED > NM_API_VERSION
+#error "NM_VERSION_MIN_REQUIRED must be <= NM_API_VERSION"
 #endif
 #if NM_VERSION_MAX_ALLOWED < NM_VERSION_MIN_REQUIRED
 #error "NM_VERSION_MAX_ALLOWED must be >= NM_VERSION_MIN_REQUIRED"
@@ -146,10 +146,32 @@
 # define NM_AVAILABLE_IN_1_10
 #endif
 
-#if NM_VERSION_MAX_ALLOWED < NM_VERSION_1_10_2
-# define NM_AVAILABLE_IN_1_10_2          G_UNAVAILABLE(1,10.2)
+#if NM_VERSION_MIN_REQUIRED >= NM_VERSION_1_12
+# define NM_DEPRECATED_IN_1_12           G_DEPRECATED
+# define NM_DEPRECATED_IN_1_12_FOR(f)    G_DEPRECATED_FOR(f)
 #else
-# define NM_AVAILABLE_IN_1_10_2
+# define NM_DEPRECATED_IN_1_12
+# define NM_DEPRECATED_IN_1_12_FOR(f)
+#endif
+
+#if NM_VERSION_MAX_ALLOWED < NM_VERSION_1_12
+# define NM_AVAILABLE_IN_1_12            G_UNAVAILABLE(1,12)
+#else
+# define NM_AVAILABLE_IN_1_12
+#endif
+
+#if NM_VERSION_MIN_REQUIRED >= NM_VERSION_1_14
+# define NM_DEPRECATED_IN_1_14           G_DEPRECATED
+# define NM_DEPRECATED_IN_1_14_FOR(f)    G_DEPRECATED_FOR(f)
+#else
+# define NM_DEPRECATED_IN_1_14
+# define NM_DEPRECATED_IN_1_14_FOR(f)
+#endif
+
+#if NM_VERSION_MAX_ALLOWED < NM_VERSION_1_14
+# define NM_AVAILABLE_IN_1_14            G_UNAVAILABLE(1,14)
+#else
+# define NM_AVAILABLE_IN_1_14
 #endif
 
 #endif  /* NM_VERSION_H */
