@@ -1,10 +1,6 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*- */
 
 /*
- * Dan Williams <dcbw@redhat.com>
- * Hicham HAOUARI <hicham.haouari@gmail.com>
- * Pantelis Koukousoulas <pktoss@gmail.com>
- *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
@@ -20,14 +16,17 @@
  * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301 USA.
  *
- * (C) Copyright 2007 - 2008 Red Hat, Inc.
+ * Copyright 2011 - 2013 Red Hat, Inc.
  */
 
+#include "nm-default.h"
+
+#include <string.h>
+
 #include "nm-setting-adsl.h"
 #include "nm-setting-ppp.h"
 #include "nm-setting-private.h"
 #include "nm-utils.h"
-#include <string.h>
 
 /**
  * SECTION:nm-setting-adsl
@@ -204,43 +203,43 @@ verify (NMSetting *setting, GSList *all_settings, GError **error)
 	NMSettingAdslPrivate *priv = NM_SETTING_ADSL_GET_PRIVATE (setting);
 
 	if (!priv->username) {
-		g_set_error (error,
-		             NM_SETTING_ADSL_ERROR,
-		             NM_SETTING_ADSL_ERROR_MISSING_PROPERTY,
-		             NM_SETTING_ADSL_USERNAME);
+		g_set_error_literal (error,
+		                     NM_SETTING_ADSL_ERROR,
+		                     NM_SETTING_ADSL_ERROR_MISSING_PROPERTY,
+		                     _("property is missing"));
+		g_prefix_error (error, "%s.%s: ", NM_SETTING_ADSL_SETTING_NAME, NM_SETTING_ADSL_USERNAME);
 		return FALSE;
 	} else if (!strlen (priv->username)) {
-		g_set_error (error,
-		             NM_SETTING_ADSL_ERROR,
-		             NM_SETTING_ADSL_ERROR_INVALID_PROPERTY,
-		             NM_SETTING_ADSL_USERNAME);
+		g_set_error_literal (error,
+		                     NM_SETTING_ADSL_ERROR,
+		                     NM_SETTING_ADSL_ERROR_INVALID_PROPERTY,
+		                     _("property is empty"));
+		g_prefix_error (error, "%s.%s: ", NM_SETTING_ADSL_SETTING_NAME, NM_SETTING_ADSL_USERNAME);
 		return FALSE;
 	}
 
-	if (priv->password && !strlen (priv->password)) {
+	if (   !priv->protocol
+	    || (   strcmp (priv->protocol, NM_SETTING_ADSL_PROTOCOL_PPPOA)
+	        && strcmp (priv->protocol, NM_SETTING_ADSL_PROTOCOL_PPPOE)
+	        && strcmp (priv->protocol, NM_SETTING_ADSL_PROTOCOL_IPOATM))){
 		g_set_error (error,
 		             NM_SETTING_ADSL_ERROR,
 		             NM_SETTING_ADSL_ERROR_INVALID_PROPERTY,
-		             NM_SETTING_ADSL_PASSWORD);
+		             _("'%s' is not a valid value for the property"),
+		             priv->protocol ?: "(null)");
+		g_prefix_error (error, "%s.%s: ", NM_SETTING_ADSL_SETTING_NAME, NM_SETTING_ADSL_PROTOCOL);
 		return FALSE;
 	}
 
-	if (strcmp (priv->protocol, NM_SETTING_ADSL_PROTOCOL_PPPOA) &&
-		strcmp (priv->protocol, NM_SETTING_ADSL_PROTOCOL_PPPOE) &&
-		strcmp (priv->protocol, NM_SETTING_ADSL_PROTOCOL_IPOATM)) {
+	if (   priv->encapsulation
+	    && (   strcmp (priv->encapsulation, NM_SETTING_ADSL_ENCAPSULATION_VCMUX)
+	        && strcmp (priv->encapsulation, NM_SETTING_ADSL_ENCAPSULATION_LLC) )) {
 		g_set_error (error,
 		             NM_SETTING_ADSL_ERROR,
 		             NM_SETTING_ADSL_ERROR_INVALID_PROPERTY,
-		             NM_SETTING_ADSL_PROTOCOL);
-		return FALSE;
-	}
-
-	if (strcmp (priv->encapsulation, NM_SETTING_ADSL_ENCAPSULATION_VCMUX) &&
-	    strcmp (priv->encapsulation, NM_SETTING_ADSL_ENCAPSULATION_LLC) ) {
-		g_set_error (error,
-		             NM_SETTING_ADSL_ERROR,
-		             NM_SETTING_ADSL_ERROR_INVALID_PROPERTY,
-		             NM_SETTING_ADSL_ENCAPSULATION);
+		             _("'%s' is not a valid value for the property"),
+		             priv->encapsulation);
+		g_prefix_error (error, "%s.%s: ", NM_SETTING_ADSL_SETTING_NAME, NM_SETTING_ADSL_ENCAPSULATION);
 		return FALSE;
 	}
 
@@ -253,7 +252,7 @@ need_secrets (NMSetting *setting)
 	NMSettingAdslPrivate *priv = NM_SETTING_ADSL_GET_PRIVATE (setting);
 	GPtrArray *secrets = NULL;
 
-	if (priv->password)
+	if (priv->password && *priv->password)
 		return NULL;
 
 	if (!(priv->password_flags & NM_SETTING_SECRET_FLAG_NOT_REQUIRED)) {
@@ -267,7 +266,6 @@ need_secrets (NMSetting *setting)
 static void
 nm_setting_adsl_init (NMSettingAdsl *setting)
 {
-	g_object_set (setting, NM_SETTING_NAME, NM_SETTING_ADSL_SETTING_NAME, NULL);
 }
 
 static void
@@ -288,6 +286,7 @@ set_property (GObject *object, guint prop_id,
               const GValue *value, GParamSpec *pspec)
 {
 	NMSettingAdslPrivate *priv = NM_SETTING_ADSL_GET_PRIVATE (object);
+	const char *str;
 
 	switch (prop_id) {
 	case PROP_USERNAME:
@@ -303,11 +302,13 @@ set_property (GObject *object, guint prop_id,
 		break;
 	case PROP_PROTOCOL:
 		g_free (priv->protocol);
-		priv->protocol = g_ascii_strdown (g_value_get_string (value), -1);
+		str = g_value_get_string (value);
+		priv->protocol = str ? g_ascii_strdown (str, -1) : NULL;
 		break;
 	case PROP_ENCAPSULATION:
 		g_free (priv->encapsulation);
-		priv->encapsulation = g_ascii_strdown (g_value_get_string (value), -1);
+		str = g_value_get_string (value);
+		priv->encapsulation = str ? g_ascii_strdown (str, -1) : NULL;
 		break;
 	case PROP_VPI:
 		priv->vpi = g_value_get_uint (value);
@@ -379,11 +380,10 @@ nm_setting_adsl_class_init (NMSettingAdslClass *setting_class)
 	 **/
 	g_object_class_install_property
 		(object_class, PROP_USERNAME,
-		 g_param_spec_string (NM_SETTING_ADSL_USERNAME,
-						  "Username",
-						  "Username used to authenticate with the pppoa service.",
-						  NULL,
-						  G_PARAM_READWRITE | NM_SETTING_PARAM_SERIALIZE));
+		 g_param_spec_string (NM_SETTING_ADSL_USERNAME, "", "",
+		                      NULL,
+		                      G_PARAM_READWRITE |
+		                      G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * NMSettingAdsl:password:
@@ -392,75 +392,71 @@ nm_setting_adsl_class_init (NMSettingAdslClass *setting_class)
 	 **/
 	g_object_class_install_property
 		(object_class, PROP_PASSWORD,
-		 g_param_spec_string (NM_SETTING_ADSL_PASSWORD,
-						  "Password",
-						  "Password used to authenticate with the pppoa service.",
-						  NULL,
-						  G_PARAM_READWRITE | NM_SETTING_PARAM_SERIALIZE | NM_SETTING_PARAM_SECRET));
+		 g_param_spec_string (NM_SETTING_ADSL_PASSWORD, "", "",
+		                      NULL,
+		                      G_PARAM_READWRITE |
+		                      NM_SETTING_PARAM_SECRET |
+		                      G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * NMSettingAdsl:password-flags:
 	 *
-	 * Flags indicating how to handle #NMSettingAdsl:password:.
+	 * Flags indicating how to handle the #NMSettingAdsl:password property.
 	 **/
-	g_object_class_install_property (object_class, PROP_PASSWORD_FLAGS,
-		 g_param_spec_uint (NM_SETTING_ADSL_PASSWORD_FLAGS,
-		                    "Password Flags",
-		                    "Flags indicating how to handle the ADSL password.",
+	g_object_class_install_property
+		(object_class, PROP_PASSWORD_FLAGS,
+		 g_param_spec_uint (NM_SETTING_ADSL_PASSWORD_FLAGS, "", "",
 		                    NM_SETTING_SECRET_FLAG_NONE,
 		                    NM_SETTING_SECRET_FLAGS_ALL,
 		                    NM_SETTING_SECRET_FLAG_NONE,
-		                    G_PARAM_READWRITE | NM_SETTING_PARAM_SERIALIZE));
+		                    G_PARAM_READWRITE |
+		                    G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * NMSettingAdsl:protocol:
 	 *
-	 * ADSL connection protocol, can be pppoa, pppoe or ipoatm.
+	 * ADSL connection protocol.  Can be "pppoa", "pppoe" or "ipoatm".
 	 **/
 	g_object_class_install_property
 		(object_class, PROP_PROTOCOL,
-		 g_param_spec_string (NM_SETTING_ADSL_PROTOCOL,
-						  "Protocol",
-						  "ADSL connection protocol.",
-						  NULL,
-						  G_PARAM_READWRITE | NM_SETTING_PARAM_SERIALIZE));
+		 g_param_spec_string (NM_SETTING_ADSL_PROTOCOL, "", "",
+		                      NULL,
+		                      G_PARAM_READWRITE |
+		                      G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * NMSettingAdsl:encapsulation:
 	 *
-	 * ADSL connection encapsulation, can be vcmux or llc.
+	 * Encapsulation of ADSL connection.  Can be "vcmux" or "llc".
 	 **/
 	g_object_class_install_property
 		(object_class, PROP_ENCAPSULATION,
-		 g_param_spec_string (NM_SETTING_ADSL_ENCAPSULATION,
-						  "Encapsulation",
-						  "Encapsulation of ADSL connection",
-						  NULL,
-						  G_PARAM_READWRITE | NM_SETTING_PARAM_SERIALIZE));
+		 g_param_spec_string (NM_SETTING_ADSL_ENCAPSULATION, "", "",
+		                      NULL,
+		                      G_PARAM_READWRITE |
+		                      G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * NMSettingAdsl:vpi:
 	 *
-	 * ADSL connection vpi.
+	 * VPI of ADSL connection
 	 **/
 	g_object_class_install_property
 		(object_class, PROP_VPI,
-		 g_param_spec_uint (NM_SETTING_ADSL_VPI,
-						  "VPI",
-						  "VPI of ADSL connection",
-						  0, 65536, 0,
-						  G_PARAM_READWRITE | NM_SETTING_PARAM_SERIALIZE));
+		 g_param_spec_uint (NM_SETTING_ADSL_VPI, "", "",
+		                    0, 65536, 0,
+		                    G_PARAM_READWRITE |
+		                    G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * NMSettingAdsl:vci:
 	 *
-	 * ADSL connection vci.
+	 * VCI of ADSL connection
 	 **/
 	g_object_class_install_property
 		(object_class, PROP_VCI,
-		 g_param_spec_uint (NM_SETTING_ADSL_VCI,
-						  "VCI",
-						  "VCI of ADSL connection",
-						  0, 65536, 0,
-						  G_PARAM_READWRITE | NM_SETTING_PARAM_SERIALIZE));
+		 g_param_spec_uint (NM_SETTING_ADSL_VCI, "", "",
+		                    0, 65536, 0,
+		                    G_PARAM_READWRITE |
+		                    G_PARAM_STATIC_STRINGS));
 }
