@@ -102,7 +102,6 @@ static void
 _ensure_registered (void)
 {
 	if (G_UNLIKELY (registered_settings == NULL)) {
-		nm_g_type_init ();
 		_nm_value_transforms_register ();
 		registered_settings = g_hash_table_new (g_str_hash, g_str_equal);
 		registered_settings_by_type = g_hash_table_new (_nm_gtype_hash, _nm_gtype_equal);
@@ -271,7 +270,7 @@ _nm_setting_lookup_setting_type_by_quark (GQuark error_quark)
 	return G_TYPE_INVALID;
 }
 
-gint
+int
 _nm_setting_compare_priority (gconstpointer a, gconstpointer b)
 {
 	guint32 prio_a, prio_b;
@@ -322,7 +321,7 @@ nm_setting_to_hash (NMSetting *setting, NMSettingHashFlags flags)
 	property_specs = g_object_class_list_properties (G_OBJECT_GET_CLASS (setting), &n_property_specs);
 
 	hash = g_hash_table_new_full (g_str_hash, g_str_equal,
-	                              (GDestroyNotify) g_free, destroy_gvalue);
+	                              g_free, destroy_gvalue);
 
 	for (i = 0; i < n_property_specs; i++) {
 		GParamSpec *prop_spec = property_specs[i];
@@ -467,7 +466,7 @@ nm_setting_duplicate (NMSetting *setting)
 	return NM_SETTING (dup);
 }
 
-static gint
+static int
 find_setting_by_name (gconstpointer a, gconstpointer b)
 {
 	NMSetting *setting = NM_SETTING (a);
@@ -617,7 +616,7 @@ nm_setting_compare (NMSetting *a,
 {
 	GParamSpec **property_specs;
 	guint n_property_specs;
-	gint same = TRUE;
+	int same = TRUE;
 	guint i;
 
 	g_return_val_if_fail (NM_IS_SETTING (a), FALSE);
@@ -1330,15 +1329,14 @@ nm_setting_get_virtual_iface_name (NMSetting *setting)
 	return NULL;
 }
 
-
 NMSettingVerifyResult
 _nm_setting_verify_deprecated_virtual_iface_name (const char *interface_name,
                                                   gboolean allow_missing,
                                                   const char *setting_name,
                                                   const char *setting_property,
                                                   GQuark error_quark,
-                                                  gint e_invalid_property,
-                                                  gint e_missing_property,
+                                                  int e_invalid_property,
+                                                  int e_missing_property,
                                                   GSList *all_settings,
                                                   GError **error)
 {
