@@ -48,7 +48,7 @@ typedef struct {
 	NM80211Mode mode;
 	guint32 max_bitrate;
 	guint8 strength;
-	gint last_seen;
+	int last_seen;
 } NMAccessPointPrivate;
 
 enum {
@@ -144,9 +144,9 @@ nm_access_point_get_ssid (NMAccessPoint *ap)
  * nm_access_point_get_frequency:
  * @ap: a #NMAccessPoint
  *
- * Gets the frequency of the access point.
+ * Gets the frequency of the access point in MHz.
  *
- * Returns: the frequency
+ * Returns: the frequency in MHz
  **/
 guint32
 nm_access_point_get_frequency (NMAccessPoint *ap)
@@ -209,9 +209,9 @@ nm_access_point_get_max_bitrate (NMAccessPoint *ap)
  * nm_access_point_get_strength:
  * @ap: a #NMAccessPoint
  *
- * Gets the current signal strength of the access point.
+ * Gets the current signal strength of the access point as a percentage.
  *
- * Returns: the signal strength
+ * Returns: the signal strength (0 to 100)
  **/
 guint8
 nm_access_point_get_strength (NMAccessPoint *ap)
@@ -233,14 +233,14 @@ nm_access_point_get_strength (NMAccessPoint *ap)
  *
  * Since: 1.2
  **/
-gint
+int
 nm_access_point_get_last_seen (NMAccessPoint *ap)
 {
 	g_return_val_if_fail (NM_IS_ACCESS_POINT (ap), -1);
 
 	return NM_ACCESS_POINT_GET_PRIVATE (ap)->last_seen;
 }
-NM_BACKPORT_SYMBOL (libnm_1_0_6, gint, nm_access_point_get_last_seen, (NMAccessPoint *ap), (ap));
+NM_BACKPORT_SYMBOL (libnm_1_0_6, int, nm_access_point_get_last_seen, (NMAccessPoint *ap), (ap));
 
 /**
  * nm_access_point_connection_valid:
@@ -367,7 +367,7 @@ nm_access_point_connection_valid (NMAccessPoint *ap, NMConnection *connection)
  * #NMDevice using nm_device_filter_connections() and finally filter that list
  * with this function.
  *
- * Returns: (transfer container) (element-type NMConnection): an array of
+ * Returns: (transfer full) (element-type NMConnection): an array of
  * #NMConnections that could be activated with the given @ap.  The array should
  * be freed with g_ptr_array_unref() when it is no longer required.
  **/
@@ -483,7 +483,6 @@ init_dbus (NMObject *object)
 	                                property_info);
 }
 
-
 static void
 nm_access_point_class_init (NMAccessPointClass *ap_class)
 {
@@ -588,7 +587,7 @@ nm_access_point_class_init (NMAccessPointClass *ap_class)
 		                      NULL,
 		                      G_PARAM_READABLE |
 		                      G_PARAM_STATIC_STRINGS));
-	
+
 	/**
 	 * NMAccessPoint:mode:
 	 *
