@@ -16,7 +16,7 @@
  * Boston, MA 02110-1301 USA.
  *
  * Copyright 2007 - 2008 Novell, Inc.
- * Copyright 2007 - 2012 Red Hat, Inc.
+ * Copyright 2007 - 2018 Red Hat, Inc.
  */
 
 #include "nm-default.h"
@@ -284,6 +284,9 @@ coerce_type (NMDeviceType type)
 	case NM_DEVICE_TYPE_UNKNOWN:
 	case NM_DEVICE_TYPE_DUMMY:
 	case NM_DEVICE_TYPE_PPP:
+	case NM_DEVICE_TYPE_WPAN:
+	case NM_DEVICE_TYPE_6LOWPAN:
+	case NM_DEVICE_TYPE_WIREGUARD:
 		return type;
 	}
 	return NM_DEVICE_TYPE_UNKNOWN;
@@ -1345,6 +1348,12 @@ get_type_name (NMDevice *device)
 		return _("Dummy");
 	case NM_DEVICE_TYPE_PPP:
 		return _("PPP");
+	case NM_DEVICE_TYPE_WPAN:
+		return _("IEEE 802.15.4");
+	case NM_DEVICE_TYPE_6LOWPAN:
+		return _("6LoWPAN");
+	case NM_DEVICE_TYPE_WIREGUARD:
+		return _("WireGuard");
 	case NM_DEVICE_TYPE_GENERIC:
 	case NM_DEVICE_TYPE_UNUSED1:
 	case NM_DEVICE_TYPE_UNUSED2:
@@ -2524,7 +2533,7 @@ nm_device_connection_compatible (NMDevice *device, NMConnection *connection, GEr
  * incompatible with the device. To get the full list of connections see
  * nm_client_get_connections().
  *
- * Returns: (transfer container) (element-type NMConnection): an array of
+ * Returns: (transfer full) (element-type NMConnection): an array of
  * #NMConnections that could be activated with the given @device.  The array
  * should be freed with g_ptr_array_unref() when it is no longer required.
  **/
