@@ -22,9 +22,11 @@
 #ifndef __NM_IP4_CONFIG_H__
 #define __NM_IP4_CONFIG_H__
 
-#include "nm-ip-config.h"
+#if !((NETWORKMANAGER_COMPILATION) & NM_NETWORKMANAGER_COMPILATION_WITH_LIBNM_PRIVATE)
+#error Cannot use this header.
+#endif
 
-G_BEGIN_DECLS
+#include "nm-ip-config.h"
 
 #define NM_TYPE_IP4_CONFIG            (nm_ip4_config_get_type ())
 #define NM_IP4_CONFIG(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), NM_TYPE_IP4_CONFIG, NMIP4Config))
@@ -42,13 +44,8 @@ typedef struct {
 
 typedef struct {
 	NMIPConfigClass parent;
-
-	/*< private >*/
-	gpointer padding[4];
 } NMIP4ConfigClass;
 
 GType nm_ip4_config_get_type (void);
 
-G_END_DECLS
-
 #endif /* __NM_IP4_CONFIG_H__ */
