@@ -33,11 +33,12 @@
 #include "nm-dbus-helpers.h"
 #include "nm-client.h"
 #include "nm-core-internal.h"
+#include "c-list/src/c-list.h"
 
 static gboolean debug = FALSE;
 #define dbgmsg(f,...) if (G_UNLIKELY (debug)) { g_message (f, ## __VA_ARGS__ ); }
 
-G_DEFINE_QUARK (nm-obj-nm, _nm_object_obj_nm);
+NM_CACHED_QUARK_FCN ("nm-obj-nm", _nm_object_obj_nm_quark)
 
 static void nm_object_initable_iface_init (GInitableIface *iface);
 static void nm_object_async_initable_iface_init (GAsyncInitableIface *iface);
@@ -77,14 +78,13 @@ typedef struct {
 	GSList *waiters;        /* if async init did not finish, users of this object need
 	                         * to defer their notifications by adding themselves here. */
 
-	GSList *notify_items;
-	guint32 notify_id;
+	CList notify_items;
+	guint notify_id;
 
-	GSList *reload_results;
 	guint reload_remaining;
-	GError *reload_error;
 
-	GSList *pending;        /* ordered list of pending property updates. */
+	CList pending;          /* ordered list of pending property updates. */
+	GPtrArray *proxies;
 } NMObjectPrivate;
 
 enum {
@@ -128,11 +128,16 @@ GDBusProxy *
 _nm_object_get_proxy (NMObject   *object,
                       const char *interface)
 {
+	NMObjectPrivate *priv;
 	GDBusInterface *proxy;
 
 	g_return_val_if_fail (NM_IS_OBJECT (object), NULL);
 
-	proxy = g_dbus_object_get_interface (NM_OBJECT_GET_PRIVATE (object)->object, interface);
+	priv = NM_OBJECT_GET_PRIVATE (object);
+	if (priv->object == NULL)
+		return NULL;
+
+	proxy = g_dbus_object_get_interface (priv->object, interface);
 	g_return_val_if_fail (proxy != NULL, NULL);
 
 	return G_DBUS_PROXY (proxy);
@@ -146,6 +151,7 @@ typedef enum {
 } NotifySignalPending;
 
 typedef struct {
+	CList lst;
 	const char *property;
 	const char *signal_prefix;
 	NotifySignalPending pending;
@@ -155,6 +161,7 @@ typedef struct {
 static void
 notify_item_free (NotifyItem *item)
 {
+	c_list_unlink_stale (&item->lst);
 	g_clear_object (&item->changed);
 	g_slice_free (NotifyItem, item);
 }
@@ -165,7 +172,8 @@ deferred_notify_cb (gpointer data)
 	NMObject *object = NM_OBJECT (data);
 	NMObjectPrivate *priv = NM_OBJECT_GET_PRIVATE (object);
 	NMObjectClass *object_class = NM_OBJECT_GET_CLASS (object);
-	GSList *props, *iter;
+	CList props;
+	CList *iter, *safe;
 
 	priv->notify_id = 0;
 
@@ -178,18 +186,18 @@ deferred_notify_cb (gpointer data)
 	 * during the g_object_notify() call separately from the property
 	 * list we're iterating.
 	 */
-	props = g_slist_reverse (priv->notify_items);
-	priv->notify_items = NULL;
+	c_list_link_after (&priv->notify_items, &props);
+	c_list_unlink (&priv->notify_items);
 
 	g_object_ref (object);
 
 	/* Emit added/removed signals first since some of our internal objects
 	 * use the added/removed signals for new object processing.
 	 */
-	for (iter = props; iter; iter = g_slist_next (iter)) {
-		NotifyItem *item = iter->data;
+	c_list_for_each (iter, &props) {
+		NotifyItem *item = c_list_entry (iter, NotifyItem, lst);
 		char buf[50];
-		gint ret = 0;
+		int ret = 0;
 
 		switch (item->pending) {
 		case NOTIFY_SIGNAL_PENDING_ADDED:
@@ -213,8 +221,8 @@ deferred_notify_cb (gpointer data)
 	}
 
 	/* Emit property change notifications second */
-	for (iter = props; iter; iter = g_slist_next (iter)) {
-		NotifyItem *item = iter->data;
+	c_list_for_each (iter, &props) {
+		NotifyItem *item = c_list_entry (iter, NotifyItem, lst);
 
 		if (item->property)
 			g_object_notify (G_OBJECT (object), item->property);
@@ -222,7 +230,9 @@ deferred_notify_cb (gpointer data)
 
 	g_object_unref (object);
 
-	g_slist_free_full (props, (GDestroyNotify) notify_item_free);
+	c_list_for_each_safe (iter, safe, &props)
+		notify_item_free (c_list_entry (iter, NotifyItem, lst));
+
 	return G_SOURCE_REMOVE;
 }
 
@@ -244,7 +254,7 @@ _nm_object_queue_notify_full (NMObject *object,
 {
 	NMObjectPrivate *priv;
 	NotifyItem *item;
-	GSList *iter;
+	CList *iter;
 
 	g_return_if_fail (NM_IS_OBJECT (object));
 	g_return_if_fail (!signal_prefix != !property);
@@ -255,8 +265,8 @@ _nm_object_queue_notify_full (NMObject *object,
 
 	property = g_intern_string (property);
 	signal_prefix = g_intern_string (signal_prefix);
-	for (iter = priv->notify_items; iter; iter = g_slist_next (iter)) {
-		item = iter->data;
+	c_list_for_each (iter, &priv->notify_items) {
+		item = c_list_entry (iter, NotifyItem, lst);
 
 		if (property && (property == item->property))
 			return;
@@ -308,7 +318,7 @@ _nm_object_queue_notify_full (NMObject *object,
 		item->pending = added ? NOTIFY_SIGNAL_PENDING_ADDED : NOTIFY_SIGNAL_PENDING_REMOVED;
 		item->changed = changed ? g_object_ref (changed) : NULL;
 	}
-	priv->notify_items = g_slist_prepend (priv->notify_items, item);
+	c_list_link_tail (&priv->notify_items, &item->lst);
 }
 
 void
@@ -318,6 +328,7 @@ _nm_object_queue_notify (NMObject *object, const char *property)
 }
 
 typedef struct {
+	CList lst_pending;
 	NMObject *self;
 	PropertyInfo *pi;
 
@@ -333,6 +344,7 @@ odata_free (gpointer data)
 {
 	ObjectCreatedData *odata = data;
 
+	c_list_unlink_stale (&odata->lst_pending);
 	g_object_unref (odata->self);
 	g_free (odata->objects);
 	g_slice_free (ObjectCreatedData, odata);
@@ -439,9 +451,10 @@ object_property_maybe_complete (NMObject *self)
 	/* The odata may hold the last reference. */
 	_nm_unused gs_unref_object NMObject *self_keep_alive = g_object_ref (self);
 	int i;
+	CList *iter, *safe;
 
-	while (priv->pending) {
-		ObjectCreatedData *odata = priv->pending->data;
+	c_list_for_each_safe (iter, safe, &priv->pending) {
+		ObjectCreatedData *odata = c_list_entry (iter, ObjectCreatedData, lst_pending);
 		PropertyInfo *pi = odata->pi;
 		gboolean different = TRUE;
 
@@ -499,16 +512,16 @@ object_property_maybe_complete (NMObject *self)
 				/* Emit added & removed */
 				for (i = 0; i < removed->len; i++) {
 					queue_added_removed_signal (self,
-								    pi->signal_prefix,
-								    g_ptr_array_index (removed, i),
-								    FALSE);
+					                            pi->signal_prefix,
+					                            g_ptr_array_index (removed, i),
+					                            FALSE);
 				}
 
 				for (i = 0; i < added->len; i++) {
 					queue_added_removed_signal (self,
-								    pi->signal_prefix,
-								    g_ptr_array_index (added, i),
-								    TRUE);
+					                            pi->signal_prefix,
+					                            g_ptr_array_index (added, i),
+					                            TRUE);
 				}
 
 				different = removed->len || added->len;
@@ -541,7 +554,6 @@ object_property_maybe_complete (NMObject *self)
 		if (--priv->reload_remaining == 0)
 			reload_complete (self, TRUE);
 
-		priv->pending = g_slist_remove (priv->pending, odata);
 		odata_free (odata);
 	}
 }
@@ -560,7 +572,7 @@ object_created (GObject *obj, const char *path, gpointer user_data)
 			object_class->object_creation_failed (odata->self, path);
 	}
 
-	odata->objects[--odata->remaining] = obj ? g_object_ref (obj) : NULL;
+	odata->objects[odata->length - odata->remaining--] = obj ? g_object_ref (obj) : NULL;
 	object_property_maybe_complete (odata->self);
 }
 
@@ -582,7 +594,8 @@ handle_object_property (NMObject *self, const char *property_name, GVariant *val
 	odata->array = FALSE;
 	odata->property_name = property_name;
 
-	priv->pending = g_slist_append (priv->pending, odata);
+	c_list_link_tail (&priv->pending, &odata->lst_pending);
+
 	priv->reload_remaining++;
 
 	path = g_variant_get_string (value, NULL);
@@ -595,8 +608,9 @@ handle_object_property (NMObject *self, const char *property_name, GVariant *val
 	object = g_dbus_object_manager_get_object (priv->object_manager, path);
 	if (!object) {
 		/* This is a server bug -- a dangling object path for an object
-		 * that does not exist. */
-		/* XXX: We've ignored this before and the server hits the condition
+		 * that does not exist.
+		 *
+		 * NOTE: We've ignored this before and the server hits the condition
 		 * more often that it should. Given we're able to recover from
 		 * ther error, let's lower the severity of the log message to
 		 * avoid unnecessarily bothering the user. This can be removed
@@ -638,7 +652,8 @@ handle_object_array_property (NMObject *self, const char *property_name, GVarian
 	odata->array = TRUE;
 	odata->property_name = property_name;
 
-	priv->pending = g_slist_append (priv->pending, odata);
+	c_list_link_tail (&priv->pending, &odata->lst_pending);
+
 	priv->reload_remaining++;
 
 	if (npaths == 0) {
@@ -829,16 +844,12 @@ demarshal_generic (NMObject *object,
 			g_free (newval);
 	} else if (pspec->value_type == G_TYPE_BYTES) {
 		GBytes **param = (GBytes **)field;
-		gconstpointer val, old_val = NULL;
-		gsize length, old_length = 0;
+		gconstpointer val;
+		gsize length = 0;
 
 		val = g_variant_get_fixed_array (value, &length, 1);
 
-		if (*param)
-			old_val = g_bytes_get_data (*param, &old_length);
-		different =    old_length != length
-		            || (   length > 0
-		                && memcmp (old_val, val, length) != 0);
+		different = !nm_utils_gbytes_equal_mem (*param, val, length);
 		if (different) {
 			if (*param)
 				g_bytes_unref (*param);
@@ -878,18 +889,18 @@ demarshal_generic (NMObject *object,
 		HANDLE_TYPE (G_VARIANT_TYPE_BYTE, guchar, g_variant_get_byte);
 	else if (pspec->value_type == G_TYPE_DOUBLE) {
 		NM_PRAGMA_WARNING_DISABLE("-Wfloat-equal")
-		HANDLE_TYPE (G_VARIANT_TYPE_DOUBLE, gdouble, g_variant_get_double);
+		HANDLE_TYPE (G_VARIANT_TYPE_DOUBLE, double, g_variant_get_double);
 		NM_PRAGMA_WARNING_REENABLE
 	} else if (pspec->value_type == G_TYPE_INT)
-		HANDLE_TYPE (G_VARIANT_TYPE_INT32, gint, g_variant_get_int32);
+		HANDLE_TYPE (G_VARIANT_TYPE_INT32, int, g_variant_get_int32);
 	else if (pspec->value_type == G_TYPE_UINT)
 		HANDLE_TYPE (G_VARIANT_TYPE_UINT32, guint, g_variant_get_uint32);
 	else if (pspec->value_type == G_TYPE_INT64)
-		HANDLE_TYPE (G_VARIANT_TYPE_INT64, gint, g_variant_get_int64);
+		HANDLE_TYPE (G_VARIANT_TYPE_INT64, gint64, g_variant_get_int64);
 	else if (pspec->value_type == G_TYPE_UINT64)
-		HANDLE_TYPE (G_VARIANT_TYPE_UINT64, guint, g_variant_get_uint64);
+		HANDLE_TYPE (G_VARIANT_TYPE_UINT64, guint64, g_variant_get_uint64);
 	else if (pspec->value_type == G_TYPE_LONG)
-		HANDLE_TYPE (G_VARIANT_TYPE_INT64, glong, g_variant_get_int64);
+		HANDLE_TYPE (G_VARIANT_TYPE_INT64, long, g_variant_get_int64);
 	else if (pspec->value_type == G_TYPE_ULONG)
 		HANDLE_TYPE (G_VARIANT_TYPE_UINT64, gulong, g_variant_get_uint64);
 	else {
@@ -938,10 +949,10 @@ _nm_object_register_properties (NMObject *object,
 
 	proxy = _nm_object_get_proxy (object, interface);
 	g_signal_connect (proxy, "g-properties-changed",
-		          G_CALLBACK (properties_changed), object);
-	g_object_unref (proxy);
+	                  G_CALLBACK (properties_changed), object);
+	g_ptr_array_add (priv->proxies, proxy);
 
-	instance = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
+	instance = g_hash_table_new_full (nm_str_hash, g_str_equal, g_free, g_free);
 	priv->property_tables = g_slist_prepend (priv->property_tables, instance);
 
 	for (tmp = (NMPropertiesInfo *) info; tmp->name; tmp++) {
@@ -953,7 +964,7 @@ _nm_object_register_properties (NMObject *object,
 		}
 
 		pi = g_malloc0 (sizeof (PropertyInfo));
-		pi->func = tmp->func ? tmp->func : demarshal_generic;
+		pi->func = tmp->func ?: demarshal_generic;
 		pi->object_type = tmp->object_type;
 		pi->field = tmp->field;
 		pi->signal_prefix = tmp->signal_prefix;
@@ -998,34 +1009,12 @@ static void
 reload_complete (NMObject *object, gboolean emit_now)
 {
 	NMObjectPrivate *priv = NM_OBJECT_GET_PRIVATE (object);
-	GSimpleAsyncResult *simple;
-	GSList *results, *iter;
-	GError *error;
 
 	if (emit_now) {
 		nm_clear_g_source (&priv->notify_id);
 		deferred_notify_cb (object);
 	} else
 		_nm_object_defer_notify (object);
-
-	results = priv->reload_results;
-	priv->reload_results = NULL;
-	error = priv->reload_error;
-	priv->reload_error = NULL;
-
-	for (iter = results; iter; iter = iter->next) {
-		simple = iter->data;
-
-		if (error)
-			g_simple_async_result_set_from_error (simple, error);
-		else
-			g_simple_async_result_set_op_res_gboolean (simple, TRUE);
-
-		g_simple_async_result_complete (simple);
-		g_object_unref (simple);
-	}
-	g_slist_free (results);
-	g_clear_error (&error);
 }
 
 GDBusObjectManager *
@@ -1042,18 +1031,19 @@ init_dbus (NMObject *object)
 }
 
 static void
-init_if (GDBusInterface *interface, gpointer user_data)
+init_if (GDBusProxy *proxy, NMObject *self)
 {
-	NMObject *self = NM_OBJECT (user_data);
-	GDBusProxy *proxy = G_DBUS_PROXY (interface);
-	gchar **props;
+	char **props;
 	char **prop;
 	GVariant *val;
-	gchar *str;
+	char *str;
+
+	nm_assert (G_IS_DBUS_PROXY (proxy));
+	nm_assert (NM_IS_OBJECT (self));
 
 	props = g_dbus_proxy_get_cached_property_names (proxy);
 
-	for (prop = props; *prop; prop++) {
+	for (prop = props; prop && *prop; prop++) {
 		val = g_dbus_proxy_get_cached_property (proxy, *prop);
 		str = g_variant_print (val, TRUE);
 		handle_property_changed (self, *prop, val);
@@ -1138,6 +1128,8 @@ init_async (GAsyncInitable *initable, int io_priority,
 	init_data = g_slice_new0 (NMObjectInitData);
 	init_data->object = self;
 	init_data->simple = g_simple_async_result_new (G_OBJECT (initable), callback, user_data, init_async);
+	if (cancellable)
+		g_simple_async_result_set_check_cancellable (init_data->simple, cancellable);
 	init_data->cancellable = cancellable ? g_object_ref (cancellable) : NULL;
 
 	interfaces = g_dbus_object_get_interfaces (priv->object);
@@ -1188,6 +1180,11 @@ nm_object_async_initable_iface_init (GAsyncInitableIface *iface)
 static void
 nm_object_init (NMObject *object)
 {
+	NMObjectPrivate *priv = NM_OBJECT_GET_PRIVATE (object);
+
+	c_list_init (&priv->notify_items);
+	c_list_init (&priv->pending);
+	priv->proxies = g_ptr_array_new ();
 }
 
 static void
@@ -1198,11 +1195,13 @@ set_property (GObject *object, guint prop_id,
 
 	switch (prop_id) {
 	case PROP_DBUS_OBJECT:
-		/* Construct only */
+		/* construct-only */
 		priv->object = g_value_dup_object (value);
+		if (!priv->object)
+			g_return_if_reached ();
 		break;
 	case PROP_DBUS_OBJECT_MANAGER:
-		/* Construct only */
+		/* construct-only */
 		priv->object_manager = g_value_dup_object (value);
 		break;
 	default:
@@ -1224,12 +1223,6 @@ get_property (GObject *object, guint prop_id,
 	case PROP_DBUS_CONNECTION:
 		g_value_set_object (value, g_dbus_object_manager_client_get_connection (G_DBUS_OBJECT_MANAGER_CLIENT (priv->object_manager)));
 		break;
-	case PROP_DBUS_OBJECT:
-		g_value_set_object (value, priv->object);
-		break;
-	case PROP_DBUS_OBJECT_MANAGER:
-		g_value_set_object (value, priv->object_manager);
-		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -1240,17 +1233,30 @@ static void
 dispose (GObject *object)
 {
 	NMObjectPrivate *priv = NM_OBJECT_GET_PRIVATE (object);
+	CList *iter, *safe;
+	guint i;
 
 	nm_clear_g_source (&priv->notify_id);
 
-	g_slist_free_full (priv->notify_items, (GDestroyNotify) notify_item_free);
-	priv->notify_items = NULL;
+	c_list_for_each_safe (iter, safe, &priv->notify_items)
+		notify_item_free (c_list_entry (iter, NotifyItem, lst));
 
 	g_slist_free_full (priv->waiters, odata_free);
 
 	g_clear_object (&priv->object);
 	g_clear_object (&priv->object_manager);
 
+	if (priv->proxies) {
+		for (i = 0; i < priv->proxies->len; i++) {
+			g_signal_handlers_disconnect_by_func (priv->proxies->pdata[i],
+			                                      properties_changed,
+			                                      object);
+			g_object_unref (priv->proxies->pdata[i]);
+		}
+		g_ptr_array_free (priv->proxies, TRUE);
+		priv->proxies = NULL;
+	}
+
 	G_OBJECT_CLASS (nm_object_parent_class)->dispose (object);
 }
 
@@ -1282,7 +1288,7 @@ nm_object_class_init (NMObjectClass *nm_object_class)
 	/* Properties */
 
 	/**
-	 * NMObject:path: (skip)
+	 * NMObject:path:
 	 *
 	 * The D-Bus object path.
 	 **/
@@ -1314,7 +1320,7 @@ nm_object_class_init (NMObjectClass *nm_object_class)
 	    (object_class, PROP_DBUS_OBJECT,
 	     g_param_spec_object (NM_OBJECT_DBUS_OBJECT, "", "",
 	                          G_TYPE_DBUS_OBJECT,
-	                          G_PARAM_READWRITE |
+	                          G_PARAM_WRITABLE |
 	                          G_PARAM_CONSTRUCT_ONLY |
 	                          G_PARAM_STATIC_STRINGS));
 
@@ -1327,7 +1333,7 @@ nm_object_class_init (NMObjectClass *nm_object_class)
 	    (object_class, PROP_DBUS_OBJECT_MANAGER,
 	     g_param_spec_object (NM_OBJECT_DBUS_OBJECT_MANAGER, "", "",
 	                          G_TYPE_DBUS_OBJECT_MANAGER,
-	                          G_PARAM_READWRITE |
+	                          G_PARAM_WRITABLE |
 	                          G_PARAM_CONSTRUCT_ONLY |
 	                          G_PARAM_STATIC_STRINGS));
 }
