@@ -224,7 +224,7 @@ verify_sender (NMSecretAgentOld *self,
 		             NM_SECRET_AGENT_ERROR,
 		             NM_SECRET_AGENT_ERROR_PERMISSION_DENIED,
 		             "Failed to request unix user: (%s) %s.",
-		             remote_error ? remote_error : "",
+		             remote_error ?: "",
 		             local->message);
 		g_free (remote_error);
 		g_error_free (local);
@@ -603,12 +603,10 @@ reg_request_cb (GObject *proxy,
 {
 	GSimpleAsyncResult *simple = user_data;
 	NMSecretAgentOld *self;
-	NMSecretAgentOldPrivate *priv;
 	GError *error = NULL;
 
 	self = NM_SECRET_AGENT_OLD (g_async_result_get_source_object (G_ASYNC_RESULT (simple)));
 	g_object_unref (self); /* drop extra ref added by get_source_object() */
-	priv = NM_SECRET_AGENT_OLD_GET_PRIVATE (self);
 
 	if (!nmdbus_agent_manager_call_register_finish (NMDBUS_AGENT_MANAGER (proxy), result, &error))
 		g_dbus_error_strip_remote_error (error);
@@ -884,7 +882,7 @@ nm_secret_agent_old_get_registered (NMSecretAgentOld *self)
 /*****************************************************************************/
 
 /**
- * nm_secret_agent_old_get_secrets:
+ * nm_secret_agent_old_get_secrets: (virtual get_secrets):
  * @self: a #NMSecretAgentOld
  * @connection: the #NMConnection for which we're asked secrets
  * @setting_name: the name of the secret setting
@@ -898,8 +896,6 @@ nm_secret_agent_old_get_registered (NMSecretAgentOld *self)
  * agent should use when performing the request, for example returning only
  * existing secrets without user interaction, or requesting entirely new
  * secrets from the user.
- *
- * Virtual: get_secrets
  */
 void
 nm_secret_agent_old_get_secrets (NMSecretAgentOld *self,
@@ -930,7 +926,7 @@ nm_secret_agent_old_get_secrets (NMSecretAgentOld *self,
 }
 
 /**
- * nm_secret_agent_old_save_secrets:
+ * nm_secret_agent_old_save_secrets: (virtual save_secrets):
  * @self: a #NMSecretAgentOld
  * @connection: a #NMConnection
  * @callback: (scope async): a callback, to be invoked when the operation is done
@@ -938,8 +934,6 @@ nm_secret_agent_old_get_secrets (NMSecretAgentOld *self,
  *
  * Asynchronously ensures that all secrets inside @connection are stored to
  * disk.
- *
- * Virtual: save_secrets
  */
 void
 nm_secret_agent_old_save_secrets (NMSecretAgentOld *self,
@@ -959,7 +953,7 @@ nm_secret_agent_old_save_secrets (NMSecretAgentOld *self,
 }
 
 /**
- * nm_secret_agent_old_delete_secrets:
+ * nm_secret_agent_old_delete_secrets: (virtual delete_secrets):
  * @self: a #NMSecretAgentOld
  * @connection: a #NMConnection
  * @callback: (scope async): a callback, to be invoked when the operation is done
@@ -967,8 +961,6 @@ nm_secret_agent_old_save_secrets (NMSecretAgentOld *self,
  *
  * Asynchronously asks the agent to delete all saved secrets belonging to
  * @connection.
- *
- * Virtual: delete_secrets
  */
 void
 nm_secret_agent_old_delete_secrets (NMSecretAgentOld *self,
