@@ -15,7 +15,7 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- * Copyright 2014 Red Hat, Inc.
+ * Copyright 2014 - 2018 Red Hat, Inc.
  */
 
 #ifndef __NM_TYPES_H__
@@ -28,8 +28,10 @@
 
 typedef struct _NMAccessPoint        NMAccessPoint;
 typedef struct _NMActiveConnection   NMActiveConnection;
+typedef struct _NMCheckpoint         NMCheckpoint;
 typedef struct _NMClient             NMClient;
 typedef struct _NMDevice             NMDevice;
+typedef struct _NMDevice6Lowpan      NMDevice6Lowpan;
 typedef struct _NMDeviceAdsl         NMDeviceAdsl;
 typedef struct _NMDeviceBond         NMDeviceBond;
 typedef struct _NMDeviceBridge       NMDeviceBridge;
@@ -53,6 +55,8 @@ typedef struct _NMDeviceVlan         NMDeviceVlan;
 typedef struct _NMDeviceVxlan        NMDeviceVxlan;
 typedef struct _NMDeviceWifi         NMDeviceWifi;
 typedef struct _NMDeviceWimax        NMDeviceWimax;
+typedef struct _NMDeviceWireGuard    NMDeviceWireGuard;
+typedef struct _NMDeviceWpan         NMDeviceWpan;
 typedef struct _NMDhcpConfig         NMDhcpConfig;
 typedef struct _NMIPConfig           NMIPConfig;
 typedef struct _NMObject             NMObject;
