@@ -14,17 +14,19 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- * Copyright 2008 - 2011 Red Hat, Inc.
+ * Copyright 2008 - 2018 Red Hat, Inc.
  */
 
 #ifndef __NM_GLIB_H__
 #define __NM_GLIB_H__
 
+/*****************************************************************************/
 
-#include <gio/gio.h>
-#include <string.h>
+#ifndef __NM_MACROS_INTERNAL_H__
+#error "nm-glib.h requires nm-macros-internal.h. Do not include this directly"
+#endif
 
-#include "gsystem-local-alloc.h"
+/*****************************************************************************/
 
 #ifdef __clang__
 
@@ -40,6 +42,8 @@
 
 #endif
 
+/*****************************************************************************/
+
 static inline void
 __g_type_ensure (GType type)
 {
@@ -54,6 +58,8 @@ __g_type_ensure (GType type)
 }
 #define g_type_ensure __g_type_ensure
 
+/*****************************************************************************/
+
 #if !GLIB_CHECK_VERSION(2,34,0)
 
 #define g_clear_pointer(pp, destroy) \
@@ -73,6 +79,12 @@ __g_type_ensure (GType type)
         }                                                                          \
     } G_STMT_END
 
+#endif
+
+/*****************************************************************************/
+
+#if !GLIB_CHECK_VERSION(2,34,0)
+
 /* These are used to clean up the output of test programs; we can just let
  * them no-op in older glib.
  */
@@ -102,6 +114,7 @@ __g_type_ensure (GType type)
 
 #endif
 
+/*****************************************************************************/
 
 #if GLIB_CHECK_VERSION (2, 35, 0)
 /* For glib >= 2.36, g_type_init() is deprecated.
@@ -112,12 +125,15 @@ __g_type_ensure (GType type)
 #define nm_g_type_init()     G_STMT_START { g_type_init (); } G_STMT_END
 #endif
 
+/*****************************************************************************/
 
 /* g_test_initialized() is only available since glib 2.36. */
 #if !GLIB_CHECK_VERSION (2, 36, 0)
 #define g_test_initialized() (g_test_config_vars->test_initialized)
 #endif
 
+/*****************************************************************************/
+
 /* g_assert_cmpmem() is only available since glib 2.46. */
 #if !GLIB_CHECK_VERSION (2, 45, 7)
 #define g_assert_cmpmem(m1, l1, m2, l2) G_STMT_START {\
@@ -132,6 +148,8 @@ __g_type_ensure (GType type)
                                         } G_STMT_END
 #endif
 
+/*****************************************************************************/
+
 /* Rumtime check for glib version. First do a compile time check which
  * (if satisfied) shortcuts the runtime check. */
 static inline gboolean
@@ -146,9 +164,11 @@ nm_glib_check_version (guint major, guint minor, guint micro)
 	               && glib_micro_version < micro));
 }
 
+/*****************************************************************************/
+
 /* g_test_skip() is only available since glib 2.38. Add a compatibility wrapper. */
 static inline void
-__nmtst_g_test_skip (const gchar *msg)
+__nmtst_g_test_skip (const char *msg)
 {
 #if GLIB_CHECK_VERSION (2, 38, 0)
 	G_GNUC_BEGIN_IGNORE_DEPRECATIONS
@@ -160,6 +180,7 @@ __nmtst_g_test_skip (const gchar *msg)
 }
 #define g_test_skip __nmtst_g_test_skip
 
+/*****************************************************************************/
 
 /* g_test_add_data_func_full() is only available since glib 2.34. Add a compatibility wrapper. */
 static inline void
@@ -184,6 +205,7 @@ __g_test_add_data_func_full (const char     *testpath,
 }
 #define g_test_add_data_func_full __g_test_add_data_func_full
 
+/*****************************************************************************/
 
 #if !GLIB_CHECK_VERSION (2, 34, 0)
 #define G_DEFINE_QUARK(QN, q_n)               \
@@ -199,6 +221,7 @@ q_n##_quark (void)                            \
 }
 #endif
 
+/*****************************************************************************/
 
 static inline gboolean
 nm_g_hash_table_replace (GHashTable *hash, gpointer key, gpointer value)
@@ -245,19 +268,21 @@ nm_g_hash_table_add (GHashTable *hash, gpointer key)
 #endif
 }
 
+/*****************************************************************************/
+
 #if !GLIB_CHECK_VERSION(2, 40, 0) || defined (NM_GLIB_COMPAT_H_TEST)
 static inline void
 _nm_g_ptr_array_insert (GPtrArray *array,
-                        gint       index_,
+                        int        index_,
                         gpointer   data)
 {
 	g_return_if_fail (array);
 	g_return_if_fail (index_ >= -1);
-	g_return_if_fail (index_ <= (gint) array->len);
+	g_return_if_fail (index_ <= (int) array->len);
 
 	g_ptr_array_add (array, data);
 
-	if (index_ != -1 && index_ != (gint) (array->len - 1)) {
+	if (index_ != -1 && index_ != (int) (array->len - 1)) {
 		memmove (&(array->pdata[index_ + 1]),
 		         &(array->pdata[index_]),
 		         (array->len - index_ - 1) * sizeof (gpointer));
@@ -265,6 +290,7 @@ _nm_g_ptr_array_insert (GPtrArray *array,
 	}
 }
 #endif
+
 #if !GLIB_CHECK_VERSION(2, 40, 0)
 #define g_ptr_array_insert(array, index, data) G_STMT_START { _nm_g_ptr_array_insert (array, index, data); } G_STMT_END
 #else
@@ -276,14 +302,15 @@ _nm_g_ptr_array_insert (GPtrArray *array,
 	} G_STMT_END
 #endif
 
+/*****************************************************************************/
 
 #if !GLIB_CHECK_VERSION (2, 40, 0)
 static inline gboolean
 _g_key_file_save_to_file (GKeyFile     *key_file,
-                          const gchar  *filename,
+                          const char   *filename,
                           GError      **error)
 {
-	gchar *contents;
+	char *contents;
 	gboolean success;
 	gsize length;
 
@@ -313,6 +340,7 @@ _g_key_file_save_to_file (GKeyFile     *key_file,
 	})
 #endif
 
+/*****************************************************************************/
 
 #if GLIB_CHECK_VERSION (2, 36, 0)
 #define g_credentials_get_unix_pid(creds, error) \
@@ -332,6 +360,7 @@ _g_key_file_save_to_file (GKeyFile     *key_file,
 	})
 #endif
 
+/*****************************************************************************/
 
 #if !GLIB_CHECK_VERSION(2, 40, 0) || defined (NM_GLIB_COMPAT_H_TEST)
 static inline gpointer *
@@ -372,6 +401,8 @@ _nm_g_hash_table_get_keys_as_array (GHashTable *hash_table,
 	})
 #endif
 
+/*****************************************************************************/
+
 #ifndef g_info
 /* g_info was only added with 2.39.2 */
 #define g_info(...)     g_log (G_LOG_DOMAIN,         \
@@ -379,6 +410,8 @@ _nm_g_hash_table_get_keys_as_array (GHashTable *hash_table,
                                __VA_ARGS__)
 #endif
 
+/*****************************************************************************/
+
 #if !GLIB_CHECK_VERSION(2, 44, 0)
 static inline gpointer
 g_steal_pointer (gpointer pp)
@@ -397,10 +430,11 @@ g_steal_pointer (gpointer pp)
   (0 ? (*(pp)) : (g_steal_pointer) (pp))
 #endif
 
+/*****************************************************************************/
 
 static inline gboolean
-_nm_g_strv_contains (const gchar * const *strv,
-                     const gchar         *str)
+_nm_g_strv_contains (const char * const *strv,
+                     const char          *str)
 {
 #if !GLIB_CHECK_VERSION(2, 44, 0)
 	g_return_val_if_fail (strv != NULL, FALSE);
@@ -420,8 +454,10 @@ _nm_g_strv_contains (const gchar * const *strv,
 }
 #define g_strv_contains _nm_g_strv_contains
 
+/*****************************************************************************/
+
 static inline GVariant *
-_nm_g_variant_new_take_string (gchar *string)
+_nm_g_variant_new_take_string (char *string)
 {
 #if !GLIB_CHECK_VERSION(2, 36, 0)
 	GVariant *value;
@@ -452,6 +488,8 @@ _nm_g_variant_new_take_string (gchar *string)
 }
 #define g_variant_new_take_string _nm_g_variant_new_take_string
 
+/*****************************************************************************/
+
 #if !GLIB_CHECK_VERSION(2, 38, 0)
 _nm_printf (1, 2)
 static inline GVariant *
@@ -481,9 +519,23 @@ _nm_g_variant_new_printf (const char *format_string, ...)
 	})
 #endif
 
+/*****************************************************************************/
+
 #if !GLIB_CHECK_VERSION (2, 56, 0)
 #define g_object_ref(Obj)      ((typeof(Obj)) g_object_ref (Obj))
 #define g_object_ref_sink(Obj) ((typeof(Obj)) g_object_ref_sink (Obj))
 #endif
 
+/*****************************************************************************/
+
+#ifndef g_autofree
+/* we still don't rely on recent glib to provide g_autofree. Hence, we continue
+ * to use our gs_* free macros that we took from libgsystem.
+ *
+ * To ease migration towards g_auto*, add a compat define for g_autofree. */
+#define g_autofree gs_free
+#endif
+
+/*****************************************************************************/
+
 #endif  /* __NM_GLIB_H__ */
